package de.duehl.algorithm.backtracking;

/*
 * Copyright 2018 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import java.util.List;

/**
 * Dieses Interface stellt einen Zustand des durch das Backtracking zu lösenden Problems dar.
 *
 * Rekursives Vorgehen:
 *     FindeLoesung(Zustand)
 *     1. wiederhole, solange es noch neue Teil-Lösungsschritte gibt:
 *        a) wähle einen neuen Teil-Lösungsschritt;
 *        b) falls Wahl lokal gültig ist:
 *             - erweitere Zustand um Wahl;
 *             - falls Zustand vollständig ist
 *                   return true; // Lösung gefunden!
 *               anderenfalls
 *                    falls FindeLoesung(erweiterter Zustand)
 *                        return true; // Lösung!
 *                    sonst
 *                        mache Wahl rückgängig; // Sackgasse (Backtracking)!
 *     2. Da es keinen neuen Teil-Lösungsschritt gibt: return false // Keine Lösung!
 *
 * Wer dies implementiert, sollte daran denken, die Daten des Zustands richtig zu kopieren und
 * nicht nur Objekte weiter zu reichen!
 *
 * @version 1.01     2018-01-02
 * @author Christian Dühl
 */

public interface AlgorithmState<Step extends AlgorithmSolutionStep> {

    /** Gibt an, ob der Zustand (lokalen) Bedingungen nach valide ist. */
    boolean isVaild();

    /** Gibt an, ob die Lösung vollständig ist. */
    boolean isComplete();

    /** Teillösungsschritt ausführen. */
    void forward(Step step);

    /** Backtracking. */
    void backward(Step step);

    /**
     * Wählt den nächsten Teilschritt aus, wobei die Liste der angegebenen Teilschritte sich
     * bereits als Falsch erwiesen haben.
     */
    Step chooseNextStep(List<Step> badSteps);

    /** Gibt an, ob bei der Auswahl des nächsten Schritts ein solcher erzeugt werden konnte. */
    boolean chooseNextStepWasSuccessfull();

}
