package de.duehl.math.graph.ged.graph.subGraph;

import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.edge.Edge;
import de.duehl.math.graph.ged.graph.vertex.Vertex;

/**
 * Diese Klasse erstellt Teilgraphen mit einer Kante.
 *
 * @version 1.02     2019-06-14
 * @author Christian Dühl
 */

public class EdgeSubGraphCreator extends SubgraphCreator {

    /**
     * Konstruktor.
     *
     * @param graph
     *            Graph zu dem Teilgraphen zu erstellen sind.
     * @param subGraphSize
     *            Größe der quadratischen Teilgraphen.
     */
    public EdgeSubGraphCreator(Graph graph, int subGraphSize) {
        super(graph, subGraphSize, subGraphSize);
    }

    /**
     * Konstruktor.
     *
     * @param graph
     *            Graph zu dem Teilgraphen zu erstellen sind.
     * @param subGraphSizeX
     *            Breite der Teilgraphen.
     * @param subGraphSizeY
     *            Höhe der Teilgraphen.
     */
    public EdgeSubGraphCreator(Graph graph, int subGraphSizeX, int subGraphSizeY) {
        super(graph, subGraphSizeX, subGraphSizeY);
    }

    /**
     * Erzeugt einen Teilgraphen der eine Kante anzeigt.
     *
     * @param edge
     *            Kante.
     */
    public Graph createOneEdgeSubGraph(Edge edge) {
        int startVertexIndex = edge.getStartVertexIndex();
        int targetVertexIndex = edge.getTargetVertexIndex();

        Vertex startVertex = graph.getVertex(startVertexIndex);
        Vertex targetVertex = graph.getVertex(targetVertexIndex);

        /* Neue Ecken erzeugen: */
        Vertex subGraphStartVertex = new Vertex(startVertex); // Deren Index wird beim Einfügen
        Vertex subGraphEndVertex = new Vertex(targetVertex);  // automatisch gesetzt!

        /* Neue Kante erzeugen: */
        Edge subGraphEntry = new Edge(edge);
        subGraphEntry.setStartVertexIndex(0);   // wichtig, nicht etwa newEndVertex.getIndex() !
        subGraphEntry.setTargetVertexIndex(1);  // der muss vor addEdge() stimmen!

        createSubGraphWithCopiedMeta();

        /* Ecken und Kanten einbauen: */
        subGraph.addVertex(subGraphStartVertex); // Index 0 - wird automatisch korrigiert
        subGraph.addVertex(subGraphEndVertex);   // Index 1 - wird automatisch korrigiert
        subGraph.addEdge(subGraphEntry);

        maximizeSubGraph(subGraph);

        return subGraph;
    }


}
