package de.duehl.math.graph.ged.ui.creation.menu;

import java.awt.event.KeyEvent;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import de.duehl.math.graph.ged.logic.Logic;
import de.duehl.math.graph.ged.ui.Gui;
import de.duehl.swing.ui.menu.MyMenuItem;
import de.duehl.swing.ui.menu.SingleMenuCreation;

/**
 * Diese Klasse stellt die Methoden zur Verfügung, um das Hilfe-Menü für die grafische Oberfläche
 * zu erstellen.
 *
 * @version 1.01     2019-05-22
 * @author Christian Dühl
 */

public class HelpMenuCreation  implements SingleMenuCreation {

    /** Verbindung zur Programmlogik. */
    private final Logic logic;

    /** Verbindung zur Gui. */
    private final Gui gui;

    /**
     * Konstruktor.
     *
     * @param logic
     *            Verbindung zur Programmlogik.
     * @param logic
     *            Verbindung zur Gui.
     */
    public HelpMenuCreation(Logic logic, Gui gui) {
        this.logic = logic;
        this.gui = gui;
    }

    /** Erzeugt das Hilfemenü. */
    @Override
    public JMenu createTheMenu() {
        JMenu menu = new JMenu("Hilfe");
        menu.setMnemonic('h');

        menu.add(createAboutMenuItem());
        menu.add(createHelpMenuItem());
        menu.add(createTodoMenuItem());
        menu.add(createChangesMenuItem());

        menu.addSeparator();

        menu.add(createWebsiteMenuItem());

        return menu;
    }

    private JMenuItem createAboutMenuItem() {
        return new MyMenuItem("Über GED ...")
                .mnemonic((char) 252) // ü ascii
                //.accelerator(KeyEvent.VK_F1, 0)
                .actionListener(e -> gui.showAbout());
    }

    private JMenuItem createHelpMenuItem() {
        return new MyMenuItem("Hilfe ...")
                .mnemonic('h')
                .accelerator(KeyEvent.VK_F1, 0)
                .actionListener(e -> gui.showHelp());
    }

    private JMenuItem createTodoMenuItem() {
        return new MyMenuItem("Todo ...")
                .mnemonic('t')
                //.accelerator(KeyEvent.VK_S, Event.CTRL_MASK)
                .actionListener(e -> gui.showTodo());
    }

    private JMenuItem createChangesMenuItem() {
        return new MyMenuItem("Historie ...")
                .mnemonic('i')
                //.accelerator(KeyEvent.VK_S, Event.CTRL_MASK)
                .actionListener(e -> gui.showChanges());
    }

    private JMenuItem createWebsiteMenuItem() {
        return new MyMenuItem("Webpage ...")
                .mnemonic('w')
                //.accelerator(KeyEvent.VK_S, Event.CTRL_MASK)
                .actionListener(e -> logic.webpage());
    }

}
