package de.duehl.math.graph.ged.ui.dialogs;

import javax.swing.JFrame;

import de.duehl.math.graph.ged.pixel.PixelPoint;
import de.duehl.math.graph.ged.ui.GuiElements;
import de.duehl.math.graph.ged.ui.graphpanel.GraphPanel;
import de.duehl.swing.ui.dialogs.base.AbstractDialogBase;

/**
 * Diese Klasse öffnet einen Dialog zur Auswahl derjenigen Kanten, die unter mehreren getroffenen
 * Kanten gelöscht werden sollen.
 *
 * @version 1.02     2019-05-22
 * @author Christian Dühl
 */

public class MouseClickLocationSetter {

    /** Der Dialog, der an die richtige Position gesetzt werden soll. */
    private final AbstractDialogBase dialog;

    /** Koordinaten, an denen der Dialog angezeigt werden soll. */
    private final PixelPoint coords;

    /** Enthält die grafischen Elemente. */
    private final GuiElements elements;

    /**
     * Konstruktor.
     *
     * @param dialog
     *            Der Dialog, der an die richtige Position gesetzt werden soll.
     * @param coords
     *            Koordinaten, an denen der Dialog angezeigt werden soll.
     * @param elements
     *            Enthält die grafischen Elemente.
     */
    public MouseClickLocationSetter(AbstractDialogBase dialog, PixelPoint coords,
            GuiElements elements) {
        this.dialog = dialog;
        this.coords = coords;
        this.elements = elements;
    }

    /** Setzt die richtige Position des Dialogs. */
    public void setLocation() {
        JFrame frame = elements.getFrame();
        int frameX = frame.getX();
        int frameY = frame.getY();

        GraphPanel graphPanel = elements.getGraphPanel();
        int panelX = graphPanel.getX();
        int panelY = graphPanel.getY();

        int menuHeight = elements.getMenuBar().getHeight();

        int pad = 19 + 10; // warum auch immer noch was fehlt...

        int x = frameX + panelX + coords.getX();
        int y = frameY + panelY + coords.getY() + menuHeight + pad;
        dialog.setLocation(x, y);
    }

}
