package de.duehl.math.graph.ged.ui.elements;

import java.awt.Component;
import java.awt.GridLayout;

import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import de.duehl.math.graph.ged.graph.edge.Edge;
import de.duehl.math.graph.ged.graph.edge.EdgeStyle;
import de.duehl.swing.ui.GuiTools;
import de.duehl.swing.ui.dialogs.base.additional.WindowRefresher;

/**
 * Diese Klasse stellt ein Element zur Auswahl des Stils einer Kante dar.
 *
 * @version 1.01     2019-05-27
 * @author Christian Dühl
 */

public class EdgeStyleChooser {

    /** Das Adjazenzelements, dessen Daten der Dialog anzeigt und ändern lässt. */
    private final Edge entry;

    /** Objekt das die Anzeige des Dialogs erneuern kann. */
    private final WindowRefresher refresher;

    /**
     * Konstruktor.
     *
     * @param entry
     *            Das Adjazenzelements, dessen Daten der Dialog anzeigt und ändern lässt.
     */
    public EdgeStyleChooser(Edge entry, WindowRefresher refresher) {
        this.entry = entry;
        this.refresher = refresher;
    }

    /** Erzeugt den Panel mit den Elementen zum Stil der Kante. */
    public Component createEdgeStylePanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(1, 0, 0, 0));
        GuiTools.createTitle("Stil der Kante", panel);

        JRadioButton edgeStyleNormalRadioButton = createEdgeStyleNormalRadioButton();
        JRadioButton edgeStyleDaschedRadioButton = createEdgeStyleDaschedRadioButton();
        JRadioButton edgeStyleDottedRadioButton = createEdgeStyleDottedRadioButton();

        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add(edgeStyleNormalRadioButton);
        buttonGroup.add(edgeStyleDaschedRadioButton);
        buttonGroup.add(edgeStyleDottedRadioButton);

        panel.add(edgeStyleNormalRadioButton);
        panel.add(edgeStyleDaschedRadioButton);
        panel.add(edgeStyleDottedRadioButton);

        return panel;
    }

    private JRadioButton createEdgeStyleNormalRadioButton() {
        JRadioButton button = new JRadioButton("normal");
        button.setSelected(entry.getStyle() == EdgeStyle.NORMAL);
        button.addActionListener(e -> setEdgeStyle(EdgeStyle.NORMAL));
        return button;
    }

    private JRadioButton createEdgeStyleDaschedRadioButton() {
        JRadioButton button = new JRadioButton("gestrichelt");
        button.setSelected(entry.getStyle() == EdgeStyle.DASHED);
        button.addActionListener(e -> setEdgeStyle(EdgeStyle.DASHED));
        return button;
    }

    private JRadioButton createEdgeStyleDottedRadioButton() {
        JRadioButton button = new JRadioButton("gepunktet");
        button.setSelected(entry.getStyle() == EdgeStyle.DOTTED);
        button.addActionListener(e -> setEdgeStyle(EdgeStyle.DOTTED));
        return button;
    }

    private void setEdgeStyle(EdgeStyle style) {
        entry.setStyle(style);
        refresher.refresh();
    }

}
