package de.duehl.math.graph.ged.ui.elements;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JToggleButton;

import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.ui.graphpanel.GraphPainter;

/**
 * Diese Klasse stellt einen Schalter zum An- oder Abwählen eines Teilgraphen dar.
 *
 * @version 1.02     2019-05-22
 * @author Christian Dühl
 */

public class SubGraphToggleButton extends JToggleButton {

    private static final long serialVersionUID = 1L;

    private static final GraphPainter GRAPH_PAINTER = new GraphPainter();

    /** Der auf dem Button darzustellende Teilgraph. */
    private final Graph subGraph;

    /**
     * Konstruktor.
     *
     * @param subGraph
     *            Der auf dem Button darzustellende Teilgraph.
     * @param subGraphSize
     *            Größe des Fensters, in dem der Graph dargestellt werden soll.
     */
    public SubGraphToggleButton(Graph subGraph, int subGraphSize) {
        this.subGraph = subGraph;

        setBackground(subGraph.getMeta().getBackgroundColor().toSwingColor());
        setPreferredSize(new Dimension(subGraphSize, subGraphSize));
    }

    @Override
    protected void paintComponent(Graphics g) {
        if (isSelected()) {
            setBackground(new Color(255, 200, 200));
        }
        else {
            setBackground(Color.WHITE);
        }

        super.paintComponent(g); // Berücksichtigt die gerade gesetzte Hintergrundfarbe

        GRAPH_PAINTER.paintGraph(g, subGraph);
    }

}
