package de.duehl.math.graph.ged.graph.subGraph;

import de.duehl.math.graph.ged.graph.Graph;
import de.duehl.math.graph.ged.graph.MetaData;
import de.duehl.math.graph.ged.graph.vertex.Vertex;

/**
 * Diese Klasse erstellt Teilgraphen.
 *
 * @version 1.02     2019-05-27
 * @author Christian Dühl
 */

abstract class SubgraphCreator {

    protected static final int PLACE_AT_BORDERS = 10;

    /** Der Graph, zu dem Teilgraphen erzeugt werden. */
    protected final Graph graph;

    /** Breite der Canvas des Teilgraphen. */
    private final int subGraphSizeX;

    /** Höhe der Canvas des Teilgraphen. */
    private final int subGraphSizeY;

    protected Graph subGraph;

    private int minX;
    private int minY;
    private int maxX;
    private int maxY;

    /**
     * Konstruktor.
     *
     * @param graph
     *            Graph zu dem Teilgraphen zu erstellen sind.
     * @param subGraphSize
     *            Größe der quadratischen Teilgraphen.
     */
    public SubgraphCreator(Graph graph, int subGraphSize) {
        this(graph, subGraphSize, subGraphSize);
    }

    /**
     * Konstruktor.
     *
     * @param graph
     *            Graph zu dem Teilgraphen zu erstellen sind.
     * @param subGraphSizeX
     *            Breite der Teilgraphen.
     * @param subGraphSizeY
     *            Höhe der Teilgraphen.
     */
    public SubgraphCreator(Graph graph, int subGraphSizeX, int subGraphSizeY) {
        this.graph = graph;
        this.subGraphSizeX = subGraphSizeX;
        this.subGraphSizeY = subGraphSizeY;
    }

    /** Neuen Graphen mit passenden Metadaten erzeugen. */
    protected void createSubGraphWithCopiedMeta() {
        subGraph = new Graph();
        MetaData newMeta = new MetaData(graph.getMeta());
        subGraph.setMeta(newMeta);
    }

    private void calculateMinimalAndMaximalCooridnates() {
        minX = 0 + PLACE_AT_BORDERS;
        minY = 0 + PLACE_AT_BORDERS;
        maxX = subGraphSizeX - PLACE_AT_BORDERS;
        maxY = subGraphSizeY - PLACE_AT_BORDERS;
    }

    protected void maximizeSubGraph(Graph newGraph) {
        calculateMinimalAndMaximalCooridnates();
        newGraph.maximizeGraphRespectingAspectRatio(minX, minY, maxX, maxY);
    }

    protected void maximizeSubGraph(Graph newGraph, Vertex startVertex) {
        calculateMinimalAndMaximalCooridnates();
        newGraph.maximizeGraphRespectingAspectRatio(minX, minY, maxX, maxY, startVertex);
    }

}
