#------------------------------------------------------------------------------
# Module      : G u i . p m
#------------------------------------------------------------------------------
# Autor       : Christian Dühl
# Created     : 18. Januar 2005
# last change : 22. Januar 2005
# Task        : Gui functions of GED
#------------------------------------------------------------------------------

package Graph::GED::Gui;

use strict;
use warnings;

use Tk;
use Tk::Balloon;
use Tk::Dialog;
use Tk::Pane;
#use Tk::ErrorDialog;

use Math::Trig   qw/atan/;    # Arcus Tangens
use Data::Dumper qw/Dumper/;  # Daten dumpen
use File::Temp   qw//;

use Graph::GED::Core qw/$Animation
                        $Graph
                        @GitterX
                        @GitterY
                        $History
                        %Linientypen
                        %Konfig
                       /;
use Graph::GED::Data;
use Graph::GED::Geom;
use Graph::GED::Historie;
use Graph::GED::Logic;
use Graph::GED::TkTools;


#------------------------------------------------------------------------------
# Derive from Exporter:
#------------------------------------------------------------------------------
use base qw/Exporter/;
our @EXPORT_OK = qw//;


#------------------------------------------------------------------------------
# Globale private Datei-Variablen:
#------------------------------------------------------------------------------
my $Debug       =        0;
use constant PI => 3.14159;


sub start () {
    #--------------------------------------------------------------------------
    # sub          : s t a r t
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Startet die GUI.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1  - 19.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    tk_start();

    $Konfig{statuszeile} = 'Willkommen bei GED';

    $Konfig{mw}->after(100, \&refresh);
    #$Konfig{mw}->after(100,
    #                    sub {
    #                         ($Graph::GED::Graph2D::MinX,
    #                          $Graph::GED::Graph2D::MinY,
    #                          $Graph::GED::Graph2D::MaxX,
    #                          $Graph::GED::Graph2D::MaxY
    #                         ) = Graph::GED::Gui::canvas_groesse();
    #                    }
    #                  );
    MainLoop();
} # sub Graph::GED::Gui::start


sub tk_start () {
    #--------------------------------------------------------------------------
    # sub          : t k _ s t a r t
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Baut das MainWidget auf.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1  - 31.01.2003 - CD - Erstellt
    # 0.0.2  - 11.02.2003 - CD - Menü umgestellt nach durchforsten der perldoc.
    #                            toplevel menu mit cascading untermenüs.
    # 0.0.3  - 18.03.2003 - CD - Neue Linientypenkürzel verwendet, lokale Vari-
    #                            able $linientyptext eingeführt.
    # 0.0.4  - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                            %Konfig.
    #                            Umstellung vieler globaler Variablen auf
    #                            %Graph.
    # 0.0.5  - 06.02.2004 - CD - Neues Menü "Ansicht".
    # 0.0.6  - 06.02.2004 - CD - Button zum Eckenmergen hinzugefügt.
    # 0.0.7  - 07.02.2004 - CD - Bindung fürs Neuzeichnen der Zeichenfläche.
    #                            Ecken- und Kantenfarbe übers Ansichtsmenü
    #                            wählbar.
    # 0.0.8  - 09.02.2004 - CD - Erstes Bearbeiten Menü auskommentiert.
    #                            Umstellungen auf die Verwendung von Graph2D.
    #                            Eckenradiuswahl ins Menü verlagert.
    #                            Versionsnummer im Titel mit anzeigen.
    # 0.0.9  - 12.02.2004 - CD - Rückgängig- und Wiederholenbuttons eingebaut.
    # 0.0.10 - 30.12.2004 - CD - Bindung von context_menu() an die rechte
    #                            Maustaste.
    # 0.0.11 - 31.12.2004 - CD - Neuen Menüeintrag für Dumper.
    # 0.0.12 - 03.01.2005 - CD - Gitterdaten liegen nun in $Graph.
    #                            Perldoc anzeigen im Hilfemenü.
    #                            Gitterfarbe ist nun wählbar.
    #                            Balloon-Help hinzugefügt.
    # 0.0.13 - 14.01.2005 - CD - Button für: Alle Kanten an Cursorposition
    #                                        löschen.
    #                            Menüeinträge zum Anzeigen der Ecken- und
    #                            Kantennummern.
    #                            Neuer Menüeintrag zum Anzeigen des Graphen im
    #                            GED2-Format.
    # 0.0.14 - 16.01.2005 - CD - Anzeigen im GED3-Format.
    #                            Buttons zum Verschieben des Graphens.
    # 0.0.15 - 17.01.2005 - CD - Button zum Zentrieren und Skalieren und
    #                            verkleinern / vergrößern.
    # 0.0.16 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #                            Perldoc-Anzeigen auf show_perldoc umgestellt.
    # 0.0.17 - 19.01.2005 - CD - Animationselemente und Funtkionalitäten.
    # 0.0.18 - 21.01.2005 - CD - Dateioperationsbutton für die Animation.
    #--------------------------------------------------------------------------

    print "tk_start\n" if $Konfig{debug};

    my $linientyptext = 'Ecke einfügen';

    #--------------------------------------------------------------------------
    # TK-Hauptfenster:
    #--------------------------------------------------------------------------
    my $mw = $Konfig{mw} =MainWindow->new();
    set_title();

    my $icon = $mw->Photo(-file   => "$FindBin::RealBin/res/ged.bmp",
                          -height => 32,
                          -width  => 32,
                         );
    $mw->Icon(-image => $icon);


    #--------------------------------------------------------------------------
    # Status-Bar:
    #--------------------------------------------------------------------------
    my $stat_f = $mw->Frame(
       #-relief      => 'ridge',
        -borderwidth => 2,
    );
    my $stat_l = $stat_f->Label(
        -relief       => 'groove', # ridge
        -textvariable => \$Konfig{statuszeile},
        -borderwidth  => 2,
       #-background   => '#d7d7d7',
       #-state        => 'disabled',
        -anchor       => 'w',
    )->pack(
        -side         => 'left',
        -expand       => 1,
        -fill         => 'x',
    );
    $stat_f->Label(
        -relief       => 'groove', # ridge
        -textvariable => \$Konfig{changed},
        -borderwidth  => 2,
       #-background   => '#d7d7d7',
       #-state        => 'disabled',
        -width        => 1,
    )->pack(
        -side         => 'left',
        -expand       => 0,
        -fill         => 'x',
    );

    #--------------------------------------------------------------------------
    # Menü:
    #--------------------------------------------------------------------------
    my $mf = $mw->Frame(#-relief      => 'ridge',
                        -borderwidth => 2,
                       );#´´


    # Menu-Bar:
    $Konfig{menu} = $mf->Menu(
        -type => 'menubar',
        -font => $Konfig{menufont},
    );

    $mw->configure(-menu => $Konfig{menu});

    # Einträge in der Menu-Bar:
    $Konfig{menu}->cascade(
        -label     => 'Datei',
        -underline => 0,
        -font      => $Konfig{menufont},
    );
    $Konfig{menu}->cascade(
        -label     => 'Bearbeiten',
        -underline => 0,
        -font      => $Konfig{menufont},
    );
    $Konfig{menu}->cascade(
        -label     => 'Ansicht',
        -underline => 0,
        -font      => $Konfig{menufont},
    );
    $Konfig{menu}->cascade(
        -label     => 'Manipulationen',
        -underline => 0,
        -font      => $Konfig{menufont},
    );
    $Konfig{menu}->cascade(
        -label     => 'Interna',
        -underline => 0,
        -font      => $Konfig{menufont},
    );
    $Konfig{menu}->separator(); # Rest rechtsbündig
    #$Konfig{menu}->Tk::Menu::separator(); # Rest rechtsbündig
    #($Konfig{menu})->separator(); # Rest rechtsbündig
    #$Konfig{menu}->cascade(-label     => '-');
    $Konfig{menu}->cascade(
        -label     => 'Hilfe',
        -underline => 0,
        -font      => $Konfig{menufont},
    );

    # Dateimenü:
    add_file_menu();


    # Bearbeitenmenü:
    my $menu_bearbeiten = $Konfig{menu}
        ->Menu(-font      => $Konfig{menufont},
               -tearoff   => 0,
               );
    $menu_bearbeiten->cascade(-label     => 'Kreise',
                              -underline => 0,
                              -font      => $Konfig{menufont},
                             );
    $menu_bearbeiten->cascade(-label     => 'Vollständige Graphen',
                              -underline => 0,
                              -font      => $Konfig{menufont},
                             );
    $menu_bearbeiten->command(-label     => "Mittelpunkt einfügen",
                              -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'M' ],
                              -font     => $Konfig{menufont},
                             );
    my $menu_bearbeiten_kreise =
    $menu_bearbeiten
        ->Menu(-font      => $Konfig{menufont},
               -tearoff   => 0,
               -menuitems => [ ["command" => "Dreieck einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'C3' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Viereck einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'C4' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Fünfeck einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'C5' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Sechseck einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'C6' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Siebeneck einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'C7' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Achteck einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'C8' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Neuneck einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'C9' ],
                                -font     => $Konfig{menufont},
                               ],
                               '-',
                               ["command" => "Zwanzigeck einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'C20' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Dreisigeck einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'C30' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Vierzigeck einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'C40' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Fünfzigeck einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'C50' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "Neununneunzigeck einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'C99' ],
                                -font     => $Konfig{menufont},
                               ],
                             ],
              );
    $menu_bearbeiten->entryconfigure('Kreise',
                                     -menu => $menu_bearbeiten_kreise,
                                    );
    my $menu_bearbeiten_vgraphen = $menu_bearbeiten
        ->Menu(-font      => $Konfig{menufont},
               -tearoff   => 0,
               -menuitems => [ ["command" => "K_3 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K3' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_4 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K4' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_5 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K5' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_6 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K6' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_7 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K7' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_8 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K8' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_9 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K9' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_10 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K10' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_11 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K11' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_12 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K12' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_13 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K13' ],
                                -font     => $Konfig{menufont},
                               ],
                               '-',
                               ["command" => "K_20 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K20' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_30 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K30' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_40 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K40' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_50 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K50' ],
                                -font     => $Konfig{menufont},
                               ],
                               ["command" => "K_99 einfügen",
                                -command  => [ \&Graph::GED::Logic::struktur_einfuegen, 'K99' ],
                                -font     => $Konfig{menufont},
                               ],
                             ],
              );
    $menu_bearbeiten->entryconfigure('Vollständige Graphen',
                                     -menu => $menu_bearbeiten_vgraphen,
                                     );

    $Konfig{menu}->entryconfigure('Bearbeiten', -menu => $menu_bearbeiten);

    # Ansichtmenü:
    my $menu_ansicht = $Konfig{menu}
        ->Menu(-font      => $Konfig{menufont},
               -tearoff   => 0,
               -menuitems => [ ['checkbutton' => "Gitternetz anzeigen",
                                -variable     => \$Graph->{gitterzeigen}, #!!!
                                -command      => \&refresh,
                                -underline    => 0,
                                -font         => $Konfig{menufont},
                               ],
                               [qw(cascade Gitterweite -tearoff 1 -underline 6
                                   -menuitems),
                                [map ['radiobutton' => $_,
                                      -variable => \$Graph->{gitterweite}, #!!!
                                      -command  => \&refresh,
                                     ],
                                     qw(5 10 15 20 25 30 35 40 45 50
                                        60 70 80 100),
                                ],
                               ],
                               ['checkbutton' => "Am Gitternetz einrasten",
                                -variable     => \$Graph->{rastimgitter}, #!!!
                                -command      => \&refresh,
                                -underline    => 17,
                                -font         => $Konfig{menufont},
                               ],
                               ['command'     => "Gitterfarbe wählen (kleine Auswahl)",
                                -command      => sub {
                                                     my ($x, $y) = $Konfig{mw}->geometry()
                                                         =~ /\+(\d+)\+(\d+)$/;
                                                     Graph::GED::TkTools::farbwahl(
                                                         $Konfig{mw},
                                                         $x+140, $y+155,
                                                         sub {
                                                             Graph::GED::Logic::aenderung();
                                                             $Graph->gitterfarbe($_[0]);
                                                             refresh();
                                                         },
                                                     )
                                                 },
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               ['command'     => "Gitterfarbe wählen (große Auswahl)",
                                -command      => sub {
                                    Graph::GED::Logic::aenderung();
                                                     my $farbe = $Konfig{mw}->
                                                         chooseColor(-title        => 'Gitterfarbe wählen',
                                                                     -initialcolor => $Graph->hintergrund(),
                                                                    );
                                                     $Graph->gitterfarbe($farbe) if $farbe;
                                                     refresh();
                                                 },
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               '-',
                               ['command'     => "Anzeige erneuern",
                                -command      => \&refresh,
                                -accelerator  => 'F5',
                                -font         => $Konfig{menufont},
                                -underline    => 10,
                               ],
                               '-',
                               ['checkbutton' => "Eckennummern anzeigen",
                                -variable     => \$Konfig{zeige_e_nr},
                                -command      => \&refresh,
                               #-underline    => 17,
                                -font         => $Konfig{menufont},
                               ],
                               ['checkbutton' => "Kantennummern anzeigen",
                                -variable     => \$Konfig{zeige_k_nr},
                                -command      => \&refresh,
                               #-underline    => 17,
                                -font         => $Konfig{menufont},
                               ],
                               '-',
                               ['command'     => "Hintergrundfarbe wählen (kleine Auswahl)",
                                -command      => sub {
                                                     my ($x, $y) = $Konfig{mw}->geometry()
                                                         =~ /\+(\d+)\+(\d+)$/;
                                                     Graph::GED::TkTools::farbwahl(
                                                         $Konfig{mw},
                                                         $x+140, $y+155,
                                                         sub {
                                                             Graph::GED::Logic::aenderung();
                                                             $Graph->hintergrund($_[0]);
                                                             refresh();
                                                         },
                                                     )
                                                 },
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               ['command'     => "Hintergrundfarbe wählen (große Auswahl)",
                                -command      => sub {
                                    Graph::GED::Logic::aenderung();
                                                     my $farbe = $Konfig{mw}->
                                                         chooseColor(-title        => 'Hintergrundfarbe wählen',
                                                                     -initialcolor => $Graph->hintergrund(),
                                                                    );
                                                     $Graph->hintergrund($farbe) if $farbe;
                                                     refresh();
                                                 },
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               ["command"    => "Hintergrundbild festlegen",
                                -command     => \&lade_hintergrundbild,
                                #-accelerator => 'Ctrl-L',
                                -font        => $Konfig{menufont},
                               ],
                               '-',
                               ['command'     => "Default Eckenfarbe wählen (kleine Auswahl)",
                                -command      => sub {
                                                     my ($x, $y) = $Konfig{mw}->geometry()
                                                         =~ /\+(\d+)\+(\d+)$/;
                                                     Graph::GED::TkTools::farbwahl(
                                                         $Konfig{mw},
                                                         $x+140, $y+165,
                                                         sub {
                                                             Graph::GED::Logic::aenderung();
                                                             $Graph->eckenfarbe($_[0]);
                                                             refresh();
                                                         },
                                                     )
                                                 },
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               ['command'     => "Default Kantenfarbe wählen (kleine Auswahl)",
                                -command      => sub {
                                                     my ($x, $y) = $Konfig{mw}->geometry()
                                                         =~ /\+(\d+)\+(\d+)$/;
                                                     Graph::GED::TkTools::farbwahl(
                                                         $Konfig{mw},
                                                         $x+140, $y+175,
                                                         sub {
                                                             Graph::GED::Logic::aenderung();
                                                             $Graph->kantenfarbe($_[0]);
                                                             refresh();
                                                         },
                                                     )
                                                 },
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               '-',
                               ['command'     => "Default Eckenfarbe wählen (große Farbauswahl)",
                                -command      => \&waehle_eckenfarbe,
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               ['command'     => "Default Kantenfarbe wählen (große Farbauswahl)",
                                -command      => \&waehle_kantenfarbe,
                                -font         => $Konfig{menufont},
                                -underline    => 0,
                               ],
                               '-',
                               [cascade    => 'Default Eckenradius',
                                -tearoff   => 0,
                                -underline => 6,
                                -menuitems =>
                                [map ['radiobutton' => $_,
                                      -variable => \$Graph->{radius},
                                      # UNSCHOEN da an Getter / Setter vorbei!
                                      -command  => \&refresh,
                                     ],
                                     qw(3 4 5 6 7 8 9 10 12 14 16 18 20 25 30
                                        35 40 50 75 100),
                                ],
                               ],
                               [cascade    => 'Default Kantendicke',
                                -tearoff   => 0,
                                -underline => 6,
                                -menuitems =>
                                [map ['radiobutton' => $_,
                                      -variable => \$Graph->{kantendicke},
                                      # UNSCHOEN da an Getter / Setter vorbei!
                                      -command  => \&refresh,
                                     ],
                                     qw/1 2 3 4 5 6 7 8 9 10 11 12 14 16 17 18
                                        19 20 25 30 35 40 50 75 100/,
                                ],
                               ],
                             ],
              );
    $Konfig{menu}->entryconfigure('Ansicht', -menu => $menu_ansicht);

    # Manipulatorenmenü:
    my $menu_manip = $Konfig{menu}
        ->Menu(-font      => $Konfig{menufont},
               -tearoff   => 0,
               -menuitems => [ ["command"    => "In Liniengraphen umwandeln",
                                -command     => \&Graph::GED::Logic::erzeuge_liniengraphen,
                                -underline   => 3,
                                -font        => $Konfig{menufont},
                               ],
                               #["command"    => 'Dump eingeben',
                               # -command     => \&dump_eingeben,
                               # -font        => $Konfig{menufont},
                               #],
                             ],
              );
    $Konfig{menu}->entryconfigure('Manipulationen', -menu => $menu_manip);

    # Internamenü:
    my $menu_interna = $Konfig{menu}
        ->Menu(-font      => $Konfig{menufont},
               -tearoff   => 0,
               -menuitems => [ ["command"    => "interne Daten anzeigen (GED-Format)",
                                -command     => [ \&zeige_daten_an, '.ged'],
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "interne Daten anzeigen (GED2-Format)",
                                -command     => [ \&zeige_daten_an, '.ged2'],
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "interne Daten anzeigen (GED3-Format)",
                                -command     => [ \&zeige_daten_an, '.ged3'],
                                -accelerator => 'Ctrl-I',
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => "interne Struktur anzeigen (Dumper)",
                                -command     => \&zeige_daten_an_dumper,
                               #-accelerator => 'Ctrl-I',
                                -font        => $Konfig{menufont},
                               ],
                               ["command"    => 'Dump eingeben (GED-Format)',
                                -command     => \&dump_eingeben,
                                -font        => $Konfig{menufont},
                               ],
                             ],
              );
    $Konfig{menu}->entryconfigure('Interna', -menu => $menu_interna);

    # Hilfemenü:
    my $menu_hilfe = $Konfig{menu}->Menu(
        -font      => $Konfig{menufont},
        -tearoff   => 0,
        -menuitems => [
            ["command"    => "Hilfe",
             -command     => \&hilfe,
             -accelerator => 'F1',
             -font        => $Konfig{menufont},
            ],
            '-',
            ["command"    => "Was sind Graphen?",
             -command     => \&graph_erkl,
             -font        => $Konfig{menufont},
            ],
            ["command"    => "History",
             -command     => \&history,
             -font        => $Konfig{menufont},
            ],
            ["command"    => "Todo",
             -command     => \&todo,
             -font        => $Konfig{menufont},
            ],
            '-',
            ["command"    => "perldoc ged.pl",
             -command     => [\&show_perldoc, 'ged.pl'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Animation.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Animation.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Basis.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Basis.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Core.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Core.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Data.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Data.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Ecke.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Ecke.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Kante.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Kante.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/FHist.pm",
             -command     => [\&show_perldoc, 'Graph::GED::FHist.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Graph.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Graph.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Graph2D.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Graph2D.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Geom.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Geom.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Gui.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Gui.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Historie.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Historie.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Logic.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Logic.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Test.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Test.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Text.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Text.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/TkTools.pm",
             -command     => [\&show_perldoc, 'Graph::GED::TkTools.pm'],
             -font        => $Konfig{menufont},
            ],
            ["command"    => "perldoc Graph/GED/Logic.pm",
             -command     => [\&show_perldoc, 'Graph::GED::Logic.pm'],
             -font        => $Konfig{menufont},
            ],
            '-',
            ["command"    => "Über GED",
             -command     => \&ueber_ged,
             -font        => $Konfig{menufont},
            ],
        ],
    );
    $Konfig{menu}->entryconfigure('Hilfe', -menu => $menu_hilfe);

    #--------------------------------------------------------------------------
    # Linker Animationsrahmen:
    #--------------------------------------------------------------------------
    my $flinks = $mw->Frame(
        -relief      => 'groove',
        -borderwidth => 2,
    );
    $flinks->Label(
        -text => 'Animation',
    )->pack();
    $Konfig{animationskala} = $flinks->Scale(
        -from         => 1,
        -to           => 1,
        -label        => '',
        -orient       => 'vertical',
        -tickinterval => 1,
        -command      => \&animation_schalten,
    )->pack(
        -side   => 'top',
        -expand => 1,
        -fill   => 'y',
    );
    my $fanibut1 = $flinks->Frame(
        -relief      => 'groove',
        -borderwidth => 2,
    )->pack(
        -side        => 'top',
        -padx        => 2,
    );
    my $fanibut2 = $flinks->Frame(
        -relief      => 'groove',
        -borderwidth => 2,
    )->pack(
        -side        => 'top',
        -padx        => 2,
    );
    my $anibutplus = $fanibut1->Button(
        -text     => '+',
        -image    => $mw->Photo(-file => "$FindBin::RealBin/res/ani_plus.gif"),
        -command  =>
            sub {
                $Animation->push_graph  (new Graph::GED::Graph2D );
                $Animation->push_history(new Graph::GED::Historie);
                $Konfig{animationskala}->configure(
                    -to => $Animation->number_of_graphs()
                );
                #refresh();
            },
    )->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'left',
    );
    my $anibutminus = $fanibut1->Button(
        -text     => '-',
        -image    => $mw->Photo(-file => "$FindBin::RealBin/res/ani_minus.gif"),
        -command  =>
            sub {
                $Animation->pop_graph();
                $Animation->pop_history();
                $Konfig{animationskala}->configure(
                    -to => $Animation->number_of_graphs()
                );
                refresh();
            },
    )->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'left',
    );
    my $anibutcopy = $fanibut2->Button(
        -text           => 'cp',
        -image    => $mw->Photo(-file => "$FindBin::RealBin/res/ani_copy.gif"),
        -command  =>
            sub {
                my $g = $Graph->clone();
                my $h = new Graph::GED::Historie;
                #my $n = $Konfig{animationskala}->get();
                $Animation->push_graph  ($g); #new Graph::GED::Graph2D );
                $Animation->push_history(new Graph::GED::Historie);
                #$Animation->insert_graph_index($n, $g);
                #$Animation->insert_history_index($n, $g);
                $Konfig{animationskala}->configure(
                    -to => $Animation->number_of_graphs()
                );
                animation_schalten($Animation->number_of_graphs());
                refresh();
            },
    )->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'left',
    );
    my $anibutmanag = $fanibut2->Button(
        -text           => 'AV',
        -image    => $mw->Photo(-file => "$FindBin::RealBin/res/ani_verw.gif"),
        -command        => \&animations_verwaltung,
    )->pack(
        -expand => 1,
        -fill   => 'x',
        -side   => 'left',
    );
    # Dateibutton-Frame:
    my $fdbani = $flinks->Frame(
        -relief      => 'groove',
        -borderwidth => 2,
    )->pack(
        -side        => 'top',
        -padx        => 2,
    );
    my $fdbani2 = $flinks->Frame(
        -relief      => 'groove',
        -borderwidth => 2,
    )->pack(
        -side        => 'top',
        -padx        => 2,
    );
    my @dbilderani = (
        $mw->Photo(-file => "$FindBin::RealBin/res/datei_neu.gif"),
        $mw->Photo(-file => "$FindBin::RealBin/res/datei_oeffnen.gif"),
        $mw->Photo(-file => "$FindBin::RealBin/res/datei_speichern.gif"),
        $mw->Photo(-file => "$FindBin::RealBin/res/datei_speichern_unter.gif"),
    );
    my @dbefehleani = (
        \&Graph::GED::Data::neue_animation,
        \&lade_animation,
        \&speichere_animation,
        \&speichere_animation_als,
    );
    my @dateibuttonsani;
    for my $i (0..$#dbilderani) {
        push @dateibuttonsani, ($i<2?$fdbani:$fdbani2)->Button(
            -text    => 'text',
            -command => $dbefehleani[$i],
            -image   => $dbilderani[$i],
           #-relief  => 'flat',
        ) ->pack(
            -side => 'left',
        );
    }


    #--------------------------------------------------------------------------
    # Zeichenfläche:
    #--------------------------------------------------------------------------
    my $graf_lw = $mw->Canvas(-background => 'white',
                              -state      => 'disabled',
                              -width      => 80,
                              -height     => 25,
                              -relief     => 'flat',
                              -cursor     => 'crosshair',
                             );

    $Konfig{canvas} = $graf_lw;
    bind_start(); # <----- $Konfig{canvas}

    #--------------------------------------------------------------------------
    # Schalter:
    #--------------------------------------------------------------------------
    my $f = $mw->Frame(-relief      => 'groove',
                       -borderwidth => 2,
                      );


    my $clr_b = $f->Button(-text           => 'Alles löschen',
                           -command        => \&clear,
                          )
                    ->pack(#-expand => 1,
                           -fill   => 'x',
                           -side   => 'top',
                          );

    my $fmove = $f->Frame()
                   ->pack(-fill   => 'x',
                          -side   => 'top',
                         );
    my @movebuttons;
    push @movebuttons,
    $fmove->Button(-text    => 'nach links',
                   -command => sub {Graph::GED::Logic::aenderung();
                                    $Graph->nach_links();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/nach_links.gif"),
                  #-relief  => 'flat',
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );
    push @movebuttons,
    $fmove->Button(-text    => 'nach oben',
                   -command => sub {Graph::GED::Logic::aenderung();
                                    $Graph->nach_oben();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/nach_oben.gif"),
                  #-relief  => 'flat',
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );
    push @movebuttons,
    $fmove->Button(-text    => 'nach unten',
                   -command => sub {Graph::GED::Logic::aenderung();
                                    $Graph->nach_unten();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/nach_unten.gif"),
                  #-relief  => 'flat',
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );
    push @movebuttons,
    $fmove->Button(-text    => 'nach rechts',
                   -command => sub {Graph::GED::Logic::aenderung();
                                    $Graph->nach_rechts();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/nach_rechts.gif"),
                  #-relief  => 'flat',
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );

    my $fskal = $f->Frame()
                   ->pack(-fill   => 'x',
                          -side   => 'top',
                         );
    my @skalbuttons;
    push @skalbuttons,
    $fskal->Button(-text    => 'zentrieren',
                   -command => sub {Graph::GED::Logic::aenderung();
                                    $Graph->zentrieren();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/zentrieren.gif"),
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );
    push @skalbuttons,
    $fskal->Button(-text    => 'zentrieren plus',
                   -command => sub {Graph::GED::Logic::aenderung();
                                    $Graph->zentrieren_skalieren();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/zentrieren_plus.gif"),
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );
    push @skalbuttons,
    $fskal->Button(-text    => 'verkleinern',
                   -command => sub {Graph::GED::Logic::aenderung();
                                    $Graph->kleiner();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/kleiner.gif"),
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );
    push @skalbuttons,
    $fskal->Button(-text    => 'vergroessern',
                   -command => sub {Graph::GED::Logic::aenderung();
                                    $Graph->groesser();
                                    refresh();
                                   },
                   -image   => $mw->Photo(-file => "$FindBin::RealBin/res/groesser.gif"),
                  )
            ->pack(-side    => 'left',
                   -fill    => 'x',
                  );

    #my $skala = $f->Scale(-from     => 1,
    #                      -to       => 50,
    #                      -label    => 'Eckenradius',
    #                      -orient   => 'horizontal',
    #                      -variable => \$Graph->{radius}, ###UNSCHOEN !!!
    #                      -command  => sub {$Graph->r2d();
    #                                        refresh();
    #                                       },
    #                     )
    #               ->pack();

    my $quit_b = $f->Button(-text           => 'Beenden',
                            -command        => \&Graph::GED::Logic::beenden,
                           )
                     ->pack(#-expand => 1,
                            -fill   => 'x',
                            -side   => 'bottom',
                           );
    my $refr_b = $f->Button(-text           => 'Neu zeichnen',
                            -command        => \&refresh,
                           )
                     ->pack(#-expand => 1,
                            -fill   => 'x',
                            -side   => 'bottom',
                           );

    #--------------------------------------------------------------------------
    # Bilder-Buttons:
    #--------------------------------------------------------------------------
    my $fb = $mw->Frame();

    # Dateibutton-Frame:
    my $fdb = $fb->Frame(-relief      => 'groove',
                         -borderwidth => 2,
                        )
                  ->pack(-side        => 'left',
                         -padx        => 2,
                        );
    my @dbilder = ($mw->Photo(-file => "$FindBin::RealBin/res/datei_neu.gif"),
                   $mw->Photo(-file => "$FindBin::RealBin/res/datei_oeffnen.gif"),
                   $mw->Photo(-file => "$FindBin::RealBin/res/datei_speichern.gif"),
                   $mw->Photo(-file => "$FindBin::RealBin/res/datei_speichern_unter.gif"),
                  );
    my @dbefehle = (\&Graph::GED::Data::neue_datei,
                    \&lade_datei,
                    \&speichere_datei,
                    \&speichere_datei_als,
                  );
    my @dateibuttons;
    for my $i (0..$#dbilder) {
        push @dateibuttons,
        $fdb->Button(-text    => 'text',
                     -command => $dbefehle[$i],
                     -image   => $dbilder[$i],
                     -relief  => 'flat',
                    )
              ->pack(-side => 'left');
    }

    # Rückgängig-/Wiederholenbutton-Frame:
    my $frw = $fb->Frame(-relief      => 'groove',
                         -borderwidth => 2,
                        )
                  ->pack(-side        => 'left',
                         -padx        => 2,
                        );
    $Konfig{rueck_btn} = $frw->
        Button(-text    => 'text',
               -command => \&Graph::GED::Logic::rueckgaengig,
               -image   => $mw->Photo(-file => "$FindBin::RealBin/res/rueckgaengig.gif"),
               -relief  => 'flat',
              )
        ->pack(-side => 'left');
    $Konfig{wieder_btn} = $frw->
        Button(-text    => 'text',
               -command => \&Graph::GED::Logic::wiederholung,
               -image   => $mw->Photo(-file => "$FindBin::RealBin/res/wiederholung.gif"),
               -relief  => 'flat',
              )
        ->pack(-side => 'left');
    $Konfig{rueck_btn}->configure(-state => 'disabled');
    $Konfig{wieder_btn}->configure(-state => 'disabled');

    # Editbutton-Frame:
    my $feb1 = $fb->Frame(-relief      => 'groove',   # erzeugen
                          -borderwidth => 2,
                         )
                   ->pack(-side        => 'left',
                          -padx        => 2,
                         );
    my $feb2 = $fb->Frame(-relief      => 'groove',   # modifizieren
                          -borderwidth => 2,
                         )
                   ->pack(-side => 'left',
                          -padx        => 2,
                         );
    my $feb3 = $fb->Frame(-relief      => 'groove',  # löschen
                          -borderwidth => 2,
                         )
                   ->pack(-side => 'left',
                          -padx        => 2,
                         );
    my @bilder = ($mw->Photo(-file => "$FindBin::RealBin/res/ecke_neu.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/ecke_weg.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/ecke_mergen.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/ecke_verschieben.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/ecke_verschieben2.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/kante.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/kante_loeschen.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/kanten_loeschen.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/kante_hin.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/kante_zurueck.gif"),
                  $mw->Photo(-file => "$FindBin::RealBin/res/kante_beide.gif"),
                 );
    my @frames = ($feb1, $feb3, $feb3, $feb2, $feb2, $feb1, $feb3, $feb3, $feb1, $feb1, $feb1);
    my @buttons = ();
    my @befehle = (sub {$Konfig{linientyp}     = 'e_ins';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                            #$_->configure(-relief => 'flat');
                        }
                        $buttons[0]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() == 0) {
                            print "\a";
                            $Konfig{statuszeile} = "Keine Ecken zum Löschen da!";
                            return;
                        }
                        $Konfig{linientyp}     = 'e_del';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[1]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() == 0) {
                            print "\a";
                            $Konfig{statuszeile} = "Keine Ecken zum Mergen da!";
                            return;
                        }
                        $Konfig{linientyp}     = 'e_mer';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        $_->configure(-relief => 'raised') for @buttons;
                        $buttons[2]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() == 0) {
                            print "\a";
                            $Konfig{statuszeile} = "Keine Ecken zum Verschieben da!";
                            return;
                        }
                        $Konfig{linientyp}     = 'e_mv1';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[3]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() == 0) {
                            print "\a";
                            $Konfig{statuszeile} = "Keine Ecken zum Verschieben da!";
                            return;
                        }
                        $Konfig{linientyp}     = 'e_mv2';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[4]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() < 2) {
                            print "\a";
                            $Konfig{statuszeile} = "Zu wenig Ecken vorhanden!";
                            return;
                        }
                        $Konfig{linientyp}     = 'k_ins';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[5]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->m() == 0) {
                            print "\a";
                            $Konfig{statuszeile} = "Keine Kanten zum Löschen da!";
                            return;
                        }
                        $Konfig{linientyp}     = 'k_del';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[6]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->m() == 0) {
                            print "\a";
                            $Konfig{statuszeile} = "Keine Kanten zum Löschen da!";
                            return;
                        }
                        $Konfig{linientyp}     = 'k_dla';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[7]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() < 2) {
                            print "\a";
                            $Konfig{statuszeile} = "Zu wenig Ecken vorhanden!";
                            return;
                        }
                        $Konfig{linientyp}     = 'k_inh';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[8]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() < 2) {
                            print "\a";
                            $Konfig{statuszeile} = "Zu wenig Ecken vorhanden!";
                            return;
                        }
                        $Konfig{linientyp}     = 'k_inz';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[9]->configure(-relief => 'sunken');
                       },
                   sub {if ($Graph->n() < 2) {
                            print "\a";
                            $Konfig{statuszeile} = "Zu wenig Ecken vorhanden!";
                            return;
                        }
                        $Konfig{linientyp}     = 'k_inb';
                        $linientyptext = $Linientypen{$Konfig{linientyp}};
                        for (@buttons) {
                            $_->configure(-relief => 'raised');
                        }
                        $buttons[10]->configure(-relief => 'sunken');
                       },
                  );
    for my $i (0..$#bilder) {
        push @buttons, $frames[$i]->Button(-text    => 'text',
                                           -command => $befehle[$i],
                                           -image   => $bilder[$i],
                                           -relief  => 'raised',
                                           #-relief  => 'flat',
                                          )
                                    ->pack(-side => 'left');
    }
    $buttons[0]->configure(-relief => 'sunken');

    # Editbutton-Erklärtext-Frame:
    my $febt = $fb->Frame(-relief      => 'groove',
                          -borderwidth => 2,
                         )
                   ->pack(-side   => 'left',
                          -padx   => 2,
                          -ipady  => 4,
                          -expand => 1,
                          -fill   => 'x',
                         );
    $febt->Label(-textvariable => \$linientyptext,
                 -foreground   => 'blue',
                )
          ->pack(-side => 'left');

    #--------------------------------------------------------------------------
    # Elemente packen:
    #--------------------------------------------------------------------------
    $stat_f->pack(
        -side   => 'bottom',
       #-expand => 1,
        -fill   => 'x',
       #-anchor => 'w',
    );
    $mf->pack(
        -side   => 'top',
       #-expand => 1,
        -fill   => 'x',
        -anchor => 'n',
    );
    $fb->pack(
        -side   => 'top',
       #-expand => 1,
        -fill   => 'x',
    );
    $flinks->pack(
        -side   => 'left',
       #-expand => 1,
        -fill   => 'both',
    );
    
    $graf_lw->pack(
        -side   => 'left',
        -expand => 1,
        -fill   => 'both',
    );
    $f->pack(
        -side   => 'left',
       #-expand => 1,
        -fill   => 'y',
    );

    #--------------------------------------------------------------------------
    # Balloon-Widget erzeugen:
    #--------------------------------------------------------------------------
    {
        my $b = $mw->Balloon(-statusbar => $stat_l);
        $b->attach(
            $dateibuttons[0],
            -state => 'status',
            -msg   => "Der aktuelle Graph wird gelöscht und ein neuer leerer Graph erzeugt.",
        );
        $b->attach(
            $dateibuttons[1],
            -state => 'status',
            -msg   => "Öffnet eine Graphendatei und lädt den Graphen in den Editor.",
        );
        $b->attach(
            $dateibuttons[2],
            -state => 'status',
            -msg   => "Speichert den Graphen.",
        );
        $b->attach(
            $dateibuttons[3],
            -state => 'status',
            -msg   => "Speichert den Graphen unter einem anderen Namen oder in einem anderen Format.",
        );

        $b->attach(
            $Konfig{rueck_btn},
            -state => 'status',
            -msg   => "Letzte Änderung rückgängig machen.",
        );
        $b->attach(
            $Konfig{wieder_btn},
            -state => 'status',
            -msg   => "Rückgängig gemachte Änderung wiederholen.",
        );

        $b->attach(
            $buttons[0],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Einfügen neuer Ecken.",
        );
        $b->attach(
            $buttons[1],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Löschen von Ecken.",
        );
        $b->attach(
            $buttons[2],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Mergen neuer Ecken.",
        );
        $b->attach(
            $buttons[3],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Verschieben neuer Ecken (Pfeilanzeige).",
        );
        $b->attach(
            $buttons[4],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Verschieben neuer Ecken (Anzeige aller verschobenen Kanten).",
        );
        $b->attach(
            $buttons[5],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Einfügen neuer normaler Kanten.",
        );
        $b->attach(
            $buttons[6],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Löschen von Kanten.",
        );
        $b->attach(
            $buttons[7],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Löschen von allen Kanten in der Nähe.",
        );
        $b->attach(
            $buttons[8],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Einfügen neuer Kanten hin.",
        );
        $b->attach(
            $buttons[9],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Einfügen neuer Kanten zurück.",
        );
        $b->attach(
            $buttons[10],
            -state => 'status',
            -msg   => "Schaltet in den Modus zum Einfügen neuer Kanten hin und zurück.",
        );

#        $b->attach(
#            $mb,
#            -msg => 'Press and hold this button to see the menu.'
#        );
#        $b->attach(
#            $menu,
#           #-state           => 'status',
#            -balloonposition => 'mouse',
#            -msg             => [
#                'Use this to tear off the menu.',
#                 'This is a checkbox entry.',
#                 'cascade', # Cascade entry (ignored by Balloon)
#                 'separator', # Separator: never active so no message will be displayed for this entry.
#                 'This is a command entry - it will close this window.',
#            ],
#        );
        $b->attach(
            $clr_b,
            -state      => 'status',
            -balloonmsg => "Der aktuelle\nGraph wird\ngelöscht",
            -statusmsg  => "Der aktuelle Graph wird gelöscht",
        );
        $b->attach(
            $quit_b,
            -state      => 'status',
            -balloonmsg => "Beendet\ndas\nProgramm",
            -statusmsg  => "Beendet das Programm",
        );
        $b->attach(
            $refr_b,
            -state      => 'status',
            -balloonmsg => "Zeichnet den\nGraphen neu",
            -statusmsg  => "Zeichnet den Graphen neu",
        );


        $b->attach(
            $movebuttons[0],
            -state => 'status',
            -msg   => "Bewegt den Graphen um die Gitterweite nach links.",
        );
        $b->attach(
            $movebuttons[1],
            -state => 'status',
            -msg   => "Bewegt den Graphen um die Gitterweite nach oben.",
        );
        $b->attach(
            $movebuttons[2],
            -state => 'status',
            -msg   => "Bewegt den Graphen um die Gitterweite nach unten.",
        );
        $b->attach(
            $movebuttons[3],
            -state => 'status',
            -msg   => "Bewegt den Graphen um die Gitterweite nach rechts.",
        );
        $b->attach(
            $skalbuttons[0],
            -state => 'status',
            -msg   => "Zentriert den Graphen.",
        );
        $b->attach(
            $skalbuttons[1],
            -state => 'status',
            -msg   => "Zentriert und skaliert den Graphen.",
        );
        $b->attach(
            $skalbuttons[2],
            -state => 'status',
            -msg   => "Verkleinert den Graphen.",
        );
        $b->attach(
            $skalbuttons[3],
            -state => 'status',
            -msg   => "Vergrößert den Graphen.",
        );

        $b->attach(
            $anibutplus,
            -state => 'status',
            -msg   => "Erweitert die Animation um einen weiteren leeren Graphen am Ende.",
        );
        $b->attach(
            $anibutminus,
            -state => 'status',
            -msg   => "Verkürzt die Animation um den letzten Graphen am Ende.",
        );
        $b->attach(
            $anibutcopy,
            -state => 'status',
            -msg   => "Fügt den aktuellen Graphen als Kopie am Ende ein.",
        );
        $b->attach(
            $anibutmanag,
            -state => 'status',
            -msg   => "Öffnet die Animationsverwaltung.",
        );

        $b->attach(
            $dateibuttonsani[0],
            -state => 'status',
            -msg   => "Die aktuelle Animation wird gelöscht und eine neue leere Animation erzeugt.",
        );
        $b->attach(
            $dateibuttonsani[1],
            -state => 'status',
            -msg   => "Öffnet eine Animationsdatei und lädt die Animation in den Editor.",
        );
        $b->attach(
            $dateibuttonsani[2],
            -state => 'status',
            -msg   => "Speichert die Animation.",
        );
        $b->attach(
            $dateibuttonsani[3],
            -state => 'status',
            -msg   => "Speichert die Animation unter einem anderen Namen oder in einem anderen Format.",
        );

        # destroy the balloons when I am destroyed:
        #  - Balloon.pm adds bindings to all widgets which we now want to remove if we can.
        #$Konfig{balloon} = $b;
        #$mw->OnDestroy( sub { $Konfig{balloon}->destroy() } );
    }


    #--------------------------------------------------------------------------
    # Restliches:
    #--------------------------------------------------------------------------
    $mw->bind('<Control-b>', \&Graph::GED::Logic::beenden);
    $mw->bind('<Escape>',
        sub {
            $graf_lw->focus();
            clear_context_menu();
        }
    );

    $mw->bind('<F5>',        sub {$refr_b->invoke()});

    $mw->bind('<Control-n>', \&Graph::GED::Data::neue_datei);
    $mw->bind('<Control-l>', \&lade_datei);
    $mw->bind('<Control-s>', \&speichere_datei);
    $mw->bind('<Control-i>', [\&zeige_daten_an, '.ged3']);
    $mw->bind('<F1>',        \&hilfe);

    $mw->bind('<Alt-d>',     sub {$Konfig{menu}->postcascade('Datei');});
    $mw->bind('<Alt-b>',     sub {$Konfig{menu}->postcascade('Bearbeiten');});
    $mw->bind('<Alt-i>',     sub {$Konfig{menu}->postcascade('Interna');});
    $mw->bind('<Alt-h>',     sub {$Konfig{menu}->postcascade('Hilfe');});

    $mw->bind('<Prior>',         [ \&animation_schalten, '-1' ] );
    $mw->bind('<Next>',          [ \&animation_schalten, '+1' ] );
    $mw->bind('<MouseWheel>',
        [ sub {
              my $wert = -($_[1]/120);
              $wert = '+' . $wert unless $wert =~ /^-/;
              animation_schalten($wert);
          },
          Tk::Ev('D')
        ]
    );

    $Konfig{canvas}->Tk::bind("<Button-3>", [\&context_menu, Ev('x'), Ev('y'), Ev('X'), Ev('Y')]);

    # Zum Neuzeichnen für das Gitter:
    $Konfig{canvas}->CanvasBind('<Configure>', \&refresh);

    $mw->geometry('700x500'); # 600x500
} # sub Graph::GED::Gui::tk_start


sub add_file_menu () {
    #--------------------------------------------------------------------------
    # sub          : a d d _ f i l e _ m e n u
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Fügt das Dateimenü hinzu.
    #                Ausgelagert, um aktuelle bearbeitete Dateien anzuzeigen.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 17.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.3 - 09.02.2004 - CD - Ansicht erneuern auskommentiert (ist jetzt im
    #                           Menü Ansicht.
    # 0.0.4 - 10.02.2004 - CD - Module in Namensraum Graph::GED:: verschoben.
    # 0.0.5 - 03.01.2005 - CD - Inhalt löschen ist dazugekommen
    # 0.0.6 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    # 0.0.7 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    # 0.0.8 - 19.01.2005 - CD - Dateibefehle für die Animation
    #--------------------------------------------------------------------------

    print "add_file_menue\n" if $Konfig{debug};


    my $menu_datei =
    $Konfig{menu}->Menu(-font      => $Konfig{menufont},
                -tearoff   => 0,
                -menuitems => [ ["command"    => "Neu",
                                 -command     => \&Graph::GED::Data::neue_datei,
                                 -accelerator => 'Ctrl-N',
                                 -font        => $Konfig{menufont},
                                 -underline   => 0,
                                ],
                                ["command"    => "Laden",
                                 -command     => \&lade_datei,
                                 -accelerator => 'Ctrl-L',
                                 -font        => $Konfig{menufont},
                                ],
                                ["command"    => "Speichern",
                                 -command     => \&speichere_datei,
                                 -accelerator => 'Ctrl-S',
                                 -font        => $Konfig{menufont},
                                ],
                                ["command"    => "Speichern unter",
                                 -command     => \&speichere_datei_als,
                                 -font        => $Konfig{menufont},
                                ],
                                ["command"    => "Inhalt löschen",
                                 -command     => \&clear,
                                 -font        => $Konfig{menufont},
                                ],
                                '-',
                                ["command"    => "Export nach LaTeX",
                                 -command     => \&export_nach_latex,
                                 -font        => $Konfig{menufont},
                                ],
                                ["command"    => "Export nach Postscript",
                                 -command     => \&export_nach_postscript,
                                 -font        => $Konfig{menufont},
                                ],
                                '-',
                                ["command"    => "Animation Laden",
                                 -command     => \&lade_animation,
                                 -font        => $Konfig{menufont},
                                ],
                                ["command"    => "Animation Speichern",
                                 -command     => \&speichere_animation,
                                 -font        => $Konfig{menufont},
                                ],
                                ["command"    => "Animation Speichern unter",
                                 -command     => \&speichere_animation_als,
                                 -font        => $Konfig{menufont},
                                ],
                                #'-',
                                #["command"    => "Anzeige erneuern",
                                # -command     => \&refresh,
                                # -accelerator => 'F5',
                                # -font        => $Konfig{menufont},
                                #],
                              ],
               );
    $menu_datei->separator();
    for (reverse Graph::GED::FHist::array()) {
        $menu_datei->command(-label       => $_,
                             -command     =>
                              [sub {$Graph->dateiname($_[0]);
                                    clear();
                                    Graph::GED::Data::load($_[0]);
                                    refresh();
                                    set_title();
                                   }, $_
                              ],
                             #-accelerator => 'Ctrl-B', #'ESC',
                             -font        => $Konfig{menufont},
                            );
    }
    $menu_datei->separator();
    $menu_datei->command(
                         -label       => "Beenden",
                         -command     => \&Graph::GED::Logic::beenden,
                         -accelerator => 'Ctrl-B', #'ESC',
                         -font        => $Konfig{menufont},
                        );
    $Konfig{menu}->entryconfigure('Datei', -menu => $menu_datei);
} # sub Graph::GED::Gui::add_file_menu


sub bind_start () {
    #--------------------------------------------------------------------------
    # sub          : b i n d _ s t a r t
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Bindet <Button-1> an zeichen_anfang(), falls es noch keine
    #                Bindung gibt.
    #                Wenn es eine <Motion>-Bindung gibt, dann muss man dem Be-
    #                nutzer erlauben, das Zeichnen des Elements zu beenden,
    #                bevor <Button-1> erneut gebunden wird.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 31.01.2003 - CD - Erstellt
    # 0.0.2 - 04.02.2003 - CD - Canvas-Widget muss nun nicht mehr übergeben
    #                           werden.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my @bindings  = $Konfig{canvas}->Tk::bind("<Motion>");
    return if scalar @bindings;

    $Konfig{canvas}->Tk::bind(
        "<Button-1>",
        [\&zeichnen_anfang, Tk::Ev('x'), Tk::Ev('y')]
    );
} # sub Graph::GED::Gui::bind_start


sub zeichnen_anfang ($$$) {
    #--------------------------------------------------------------------------
    # sub          : z e i c h n e n _ a n f a n g
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Wird aufgerufen, wenn der Benutzer mit der Maus auf die
    #                Arbeitsfläche klickt und noch kein Zeichenvorgang statt-
    #                findet.
    #                Abhängig vom ausgewählten "Linientyp" (kann auch etwa
    #                'Ecke' sein), wird das Zeichnen der Linie begonnen (oder
    #                - wie im Fall von 'Ecke' - das Objekt gezeichnet, wenn es
    #                keiner zwei Mausklicks bedarf).
    # Parameter    : Canvas-Widget, x, y
    #                Außerdem wird die globale Variable '$Konfig{linientyp}'
    #                ausgewertet.
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1  - 31.01.2003 - CD - Erstellt
    # 0.0.2  - 03.02.2003 - CD - Eckenkoordinaten speichern
    #                            Kantenkoordinaten auf Ecken setzen und nur in
    #                            der Nähe von Ecken zu Zeichnen beginnen!
    #                            Eckenlöschen eingebaut.
    # 0.0.3  - 05.02.2003 - CD - 'Ecke verschieben' eingebaut.
    #                            'Ecke verschieben 2' eingebaut.
    # 0.0.4  - 10.02.2003 - CD - An neue Linientypenbezeichungen angepasst.
    # 0.0.5  - 18.03.2003 - CD - Neue Linientypenkürzel verwendet. Die Lang-
    #                            formen sind im Hash %Linientypen über die
    #                            Kürzel als Schlüssel zugänglich.
    # 0.0.6  - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                            %Konfig.
    #                            Umstellung vieler globaler Variablen auf
    #                            %Graph.
    # 0.0.7  - 06.02.2004 - CD - Beim Eckensetzen ggf. am Gitter einrasten.
    # 0.0.8  - 06.02.2004 - CD - Eckenmergen eingebaut.
    #                            Ecken/Kantenfunktionalitäten ausgelagert.
    # 0.0.9  - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.10 - 10.02.2004 - CD - Umstellung auf die Eckenklasse.
    # 0.0.11 - 12.02.2004 - CD - aenderung() vor Änderungen aufgerufen.
    # 0.0.12 - 03.01.2005 - CD - Gitterdaten liegen nun in $Graph.
    # 0.0.13 - 03.01.2005 - CD - aenderung() bei Verschiebungen schon zu Beginn
    #                            aufrufen.
    # 0.0.14 - 14.01.2005 - CD - Kanten löschen.
    # 0.0.15 - 17.01.2005 - CD - Kontextmenüs löschen, falls vorhanden.
    # 0.0.16 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my $w = shift; # widget
    my $x = shift; # x-Koordinate
    my $y = shift; # y-Koordinate

    print "zeichnen_anfang x='$x', y='$y', Linientyp='$Konfig{linientyp}'\n"
          if $Konfig{debug};

    #--------------------------------------------------------------------------
    # Vorarbeiten:
    #--------------------------------------------------------------------------

    clear_context_menu(); # ggf. noch offenes Kontextmenü entfernen

    # Im Fall von Linientypen Startkoordinaten auf die nächste Ecke setzen:
    if ($Konfig{linientyp} eq 'k_ins' or
        $Konfig{linientyp} eq 'k_inh' or
        $Konfig{linientyp} eq 'k_inz' or
        $Konfig{linientyp} eq 'k_inb' or
        $Konfig{linientyp} eq 'e_mv1' or
        $Konfig{linientyp} eq 'e_mv2'  )
    {
        my $e = $Graph->meint_ecke($x, $y);
        if ($e > -1 and
            ($Graph->n() > 1 or # Linienzeichnen macht erst ab 2 Ecken Sinn!
             $Graph->n() == 1 and $Konfig{linientyp} =~ /^e_mv[12]$/
            )
           )
        { 
            $Konfig{zx} = $Graph->ecke($e)->x();
            $Konfig{zy} = $Graph->ecke($e)->y();
            $Konfig{startecke} = $e;
            $Graph->kantenform($Konfig{linientyp});
        }
        else {
            print "\a";
            $Konfig{statuszeile} = 'Sie müssen an einer Ecke anfangen!';
            return;
        }
        Graph::GED::Logic::aenderung() if $Konfig{linientyp} eq 'e_mv1'
                    or $Konfig{linientyp} eq 'e_mv2';
    }
    else {
        $Konfig{zx} = $x;
        $Konfig{zy} = $y;
    }


    #--------------------------------------------------------------------------
    # Operationen, die in aendere_item() und zeichnen_ende weiter behandelt
    # werden:
    #--------------------------------------------------------------------------

    #-----------
    if ($Konfig{linientyp} eq 'k_ins') {
        $w->createLine($Konfig{zx}, $Konfig{zy}, $x, $y,
                       -width => 1,
                       -tags  => 'zeichnemichjetzt',
                       -fill  => $Graph->kantenfarbe(),
                      );
    }
    #----------->
    elsif ($Konfig{linientyp} eq 'k_inh') {
        $w->createLine($Konfig{zx}, $Konfig{zy}, $x, $y,
                       -width      => 1,
                       -tags       => 'zeichnemichjetzt',
                       -arrow      => 'last',
                       -arrowshape => [5, 7, 3],
                       -fill       => $Graph->kantenfarbe(),
                      );
    }
    # <----------
    elsif ($Konfig{linientyp} eq 'k_inz') {
        $w->createLine($Konfig{zx}, $Konfig{zy}, $x, $y,
                       -width      => 1,
                       -tags       => 'zeichnemichjetzt',
                       -arrow      => 'first',
                       -arrowshape => [5, 7, 3],
                       -fill       => $Graph->kantenfarbe(),
                      );
    }
    # <---------->
    elsif ($Konfig{linientyp} eq 'k_inb') {
        $w->createLine($Konfig{zx}, $Konfig{zy}, $x, $y,
                       -width      => 1,
                       -tags       => 'zeichnemichjetzt',
                       -arrow      => 'both',
                       -arrowshape => [5, 7, 3],
                       -fill       => $Graph->kantenfarbe(),
                      );
    }
    # * --->
    elsif ($Konfig{linientyp} eq 'e_mv1') {
        $w->createLine($Konfig{zx}, $Konfig{zy}, $x, $y,
                       -width      => 1,
                       -tags       => 'zeichnemichjetzt',
                       -width      => 1 + 2 * $Graph->radius(),
                       -arrow      => 'last',
                       -arrowshape => [
                                       5 + $Graph->radius(),
                                       7 + $Graph->radius(),
                                       3 + $Graph->radius(),
                                      ],
                       -fill       => $Graph->eckenfarbe(),
                      );
    }



    #--------------------------------------------------------------------------
    # Operationen, die hier bereits abgeschlossen werden können:
    #--------------------------------------------------------------------------

    # *
    if ($Konfig{linientyp} eq 'e_ins') {
        # falls am Gitter eingerastet werden soll, Koordinaten einrasten:
        if ($Graph->rastimgitter()) {
            ($Konfig{zx}, $Konfig{zy}) = Graph::GED::Logic::raste_ein($Konfig{zx}, $Konfig{zy});
        }

        if ($Graph->ecke_zu_nah($Konfig{zx}, $Konfig{zy})) {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind zu dicht an einer anderen Ecke!';
            return;
        }
        else {
            Graph::GED::Logic::aenderung();
            $Graph->ecke_einfuegen($Konfig{zx}, $Konfig{zy});
            zeichne_ecke($Graph->n()-1);
            $w->Tk::bind("<Motion>",  "");
            bind_start();
        }
    }
    # -*
    elsif ($Konfig{linientyp} eq 'e_del') {
        my $e = $Graph->meint_ecke($x, $y);
        if ($e > -1) {
            Graph::GED::Logic::aenderung();
            $Graph->ecke_loeschen($e);
            refresh();
        }
        else {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind nicht dicht genug an einer Ecke!';
            return;
        }
    }

    # -* merge
    elsif ($Konfig{linientyp} eq 'e_mer') {
        my $e = $Graph->meint_ecke($x, $y);
        if ($e > -1) {
            Graph::GED::Logic::aenderung();
            $Graph->ecke_mergen($e);
            refresh();
        }
        else {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind nicht dicht genug an einer Ecke!';
            return;
        }
    }
    # - ---------
    elsif ($Konfig{linientyp} eq 'k_del') {
        my $k = $Graph->meint_kante($x, $y);
        if ($k > -1) {
            Graph::GED::Logic::aenderung();
            $Graph->kante_loeschen($k);
            refresh();
        }
        else {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind nicht dicht genug an einer Kante oder '.
                           'zu dicht an zweien!';
            return;
        }
    }
    elsif ($Konfig{linientyp} eq 'k_dla') {
        my @k = $Graph->meint_kante($x, $y);
        if (@k > 0) {
            Graph::GED::Logic::aenderung();
            $Graph->kante_loeschen($_) for reverse sort {$a <=> $b} @k;
            refresh();
        }
        else {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind nicht dicht genug an einer Kante!';
            return;
        }
    }
    else {
        # Im Fall von "echten Linien" (also solchen Objekten, die eine
        # zweite Benutzerangabe zum Beenden des Zeichnens benötigen) wird
        # <Motion> an aendere_item() und <Button-1> an zeichnen_ende()
        # gebunden:
        $w->Tk::bind("<Motion>",   [\&aendere_item,  Ev('x'), Ev('y')]);
        $w->Tk::bind("<Button-1>", [\&zeichnen_ende, Ev('x'), Ev('y')]);
    }
} # sub Graph::GED::Gui::zeichnen_anfang


sub aendere_item ($$$) {
    #--------------------------------------------------------------------------
    # sub          : a e n d e r e _ i t e m
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Diese Funktion ist an <Motion> gebunden, wenn ein
    #                Zeichenvorgang läuft.
    # Parameter    : Canvas-Widget, x, y
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 31.01.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2003 - CD - 'Ecke verschieben 2' eingebaut.
    # 0.0.3 - 10.02.2003 - CD - An neue Linientypenbezeichungen angepasst.
    # 0.0.4 - 18.03.2003 - CD - Neue Linientypenkürzel verwendet.
    # 0.0.5 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.6 - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.7 - 10.02.2004 - CD - Umstellung auf die Eckenklasse.
    # 0.0.8 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my $w = shift; # widget
    my $x = shift; # x-Koordinate
    my $y = shift; # y-Koordinate

    print "aendere_item x='$x', y='$y'\n" if $Konfig{debug} > 1;

    if ($Konfig{linientyp} eq 'e_mv2') {
        my $e = $Konfig{startecke}; # hier steckt die Ecke drin!
        $Graph->ecke($e)->x($x);
        $Graph->ecke($e)->y($y);                                             #;
        refresh();
    }
    else {
        $w->coords('zeichnemichjetzt', $Konfig{zx}, $Konfig{zy}, $x, $y);
        # Hier wird auf den Tag 'zeichnemichjetzt' zugegriffen, der in
        # zeichnen_anfang() vergeben wurde.
    }
} # sub Graph::GED::Gui::aendere_item


sub zeichnen_ende ($$$) {
    #--------------------------------------------------------------------------
    # sub          : z e i c h n e n _ e n d e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Diese Funktion wird aufgerufen, wenn der Benutzer beim
    #                Zeichnen eines Objektes das zweite Mal klickt.
    #                Damit wird der Zeichenvorgang beendet, die Bindungen von
    #                <Motion> aufgehoben und die Bindung von <Button-1> durch
    #                den Aufruf von bind_start() wieder auf zeichnen_anfang()
    #                gesetzt.
    # Parameter    : Canvas-Widget, x, y
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1  - 31.01.2003 - CD - Erstellt
    # 0.0.2  - 03.02.2003 - CD - Ecke bestimmen, falls keine da, wird das Be-
    #                            enden verweigert.
    #                            Kantendaten speichern.
    # 0.0.3  - 05.02.2003 - CD - 'Ecke verschieben' eingebaut.
    #                            'Ecke verschieben 2' eingebaut.
    #                            Schleifen verhindert.
    # 0.0.4  - 10.02.2003 - CD - An neue Linientypenbezeichungen angepasst.
    # 0.0.5  - 18.03.2003 - CD - Neue Linientypenkürzel verwendet.
    #                            Bei Ecke Schnell verschieben nicht mehr auf
    #                            zu nah zur Ausgangsecke getestet.
    # 0.0.6  - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                            %Konfig.
    #                            Umstellung vieler globaler Variablen auf
    #                            %Graph.
    # 0.0.7  - 06.02.2004 - CD - Beim Eckenverschieben ggf. am Gitter ein-
    #                            rasten.
    # 0.0.8  - 06.02.2004 - CD - Ecken/Kantenfunktionalitäten ausgelagert.
    # 0.0.9  - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.10 - 10.02.2004 - CD - Umstellung auf die Eckenklasse.
    # 0.0.11 - 12.02.2004 - CD - aenderung() vor Änderungen aufgerufen.
    # 0.0.12 - 03.01.2005 - CD - Gitterdaten liegen nun in $Graph.
    # 0.0.13 - 03.01.2005 - CD - aenderung() bei Verschiebungen schon zu Beginn
    #                            aufrufen (und hier nicht mehr).
    # 0.0.14 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my $w = shift; # widget
    my $x = shift; # x-Koordinate
    my $y = shift; # y-Koordinate

    print "zeichnen_ende x='$x', y='$y'\n" if $Konfig{debug};

    if ($Konfig{linientyp} eq 'e_mv1') {
        my $e = $Konfig{startecke}; # hier steckt die Ecke drin!

        # falls am Gitter eingerastet werden soll, Koordinaten einrasten:
        if ($Graph->rastimgitter()) {
            ($x, $y) = Graph::GED::Logic::raste_ein($x, $y);
        }

        if ($Graph->ecke_zu_nah_ohne($x, $y, $e)) {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind zu dicht an einer anderen Ecke!';
            return;
        }

        else {
            #my $e = $Konfig{startecke}; # hier steckt die Ecke drin!
            print "Ecke $e wird verschoben, von (",
                   $Graph->ecke($e)->x(), ", ", $Graph->ecke($e)->y(),
                   ") auf ($x, $y).\n" if $Konfig{debug};
            $w->coords('zeichnemichjetzt', $Konfig{zx}, $Konfig{zy}, $x, $y);
            $w->delete('zeichnemichjetzt'); # dtag
            #Graph::GED::Logic::aenderung();
            $Graph->ecke($e)->x($x);
            $Graph->ecke($e)->y($y);
            refresh();
        }
    }
    elsif ($Konfig{linientyp} eq 'e_mv2') {
        my $e = $Konfig{startecke}; # hier steckt die Ecke drin!

        # falls am Gitter eingerastet werden soll, Koordinaten einrasten:
        if ($Graph->rastimgitter()) {
            ($x, $y) = Graph::GED::Logic::raste_ein($x, $y);
        }

        if ($Graph->ecke_zu_nah_ohne($x, $y, $e)) {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind zu dicht an einer anderen Ecke!';
            return;
        }
        else {
            print "Ecke $e wird verschoben, von (",
                   $Graph->ecke($e)->x(), ", ", $Graph->ecke($e)->y(),
                   ") auf ($x, $y).\n" if $Konfig{debug};
            #Graph::GED::Logic::aenderung();
            $Graph->ecke($e)->x($x);
            $Graph->ecke($e)->y($y);
            refresh();
        }
    }
    else { # Kanten
        my $e = $Graph->meint_ecke($x, $y);
        if ($e > -1) {
            if ($e == $Konfig{startecke}) {
                print "\a";
                $Konfig{statuszeile} = 'Schleifen sind im Moment nicht möglich!';
                return;
            }
            $x = $Graph->ecke($e)->x();
            $y = $Graph->ecke($e)->y();
            Graph::GED::Logic::aenderung();
            $Graph->kante_einfuegen($Konfig{startecke}, $e);
            refresh();
        }
        else {
            print "\a";
            $Konfig{statuszeile} = 'Sie sind zu nicht dicht genug an einer Ecke!';
            return;
        }

        $w->coords('zeichnemichjetzt', $Konfig{zx}, $Konfig{zy}, $x, $y);
        $w->dtag('zeichnemichjetzt');
        # Hier wird auf den Tag 'zeichnemichjetzt' zugegriffen, der in
        # zeichnen_anfang() vergeben wurde.
        # Außerdem wird der Tag jetzt von diesem Objekt entfernt, sonst würde
        # es beim Zeichenen des nächsten Objekts mit verändert.

        $Konfig{zx} = $Graph->ecke($e)->x();
        $Konfig{zy} = $Graph->ecke($e)->y();
        $Konfig{startecke} = $e;
    }

    $w->Tk::bind("<Motion>",  "");
    bind_start();
    $Konfig{statuszeile} = ''; # ok :)
} # sub Graph::GED::Gui::zeichnen_ende


sub refresh () {
    #--------------------------------------------------------------------------
    # sub          : r e f r e s h
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Alles neu zeichnen.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1  - 03.02.2003 - CD - Erstellt
    # 0.0.2  - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                            %Konfig.
    #                            Umstellung vieler globaler Variablen auf
    #                            %Graph.
    # 0.0.3  - 06.02.2004 - CD - Falls gewünscht Gitternetz einzeichnen.
    #                            Gitterlinienkoordinaten in @GitterX und
    #                            @GitterY merken.
    # 0.0.4  - 06.02.2004 - CD - canvas_groesse() aufgerufen.
    # 0.0.5  - 07.02.2004 - CD - canvas_groesse() aufgerufen und die Daten in
    #                            den Klassenvariablen $MinX, $MinY, $MaxX,
    #                            $MaxY der Klasse Graph2D gespeichert.
    # 0.0.6  - 08.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.7  - 09.02.2004 - CD - Gitterkoordinaten auch berechnen, wenn nur
    #                            eingerastet werden soll.
    # 0.0.8  - 02.01.2005 - CD - Hintergrundfarbe der Canvas setzen.
    # 0.0.9  - 03.01.2005 - CD - Gitterdaten liegen nun in $Graph.
    #                            Variable Gitterfarbe auch bei vertikalen
    #                            Linien verwenden (dort war noch die Farbe
    #                            hartcodierte eingetragen).
    # 0.0.10 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "refresh\n" if $Konfig{debug};

    #--------------------------------------------------------------------------
    # Aktuelle Größe der Zeichenfläche ermitteln:
    #--------------------------------------------------------------------------
    ($Graph::GED::Graph2D::MinX, $Graph::GED::Graph2D::MinY,
     $Graph::GED::Graph2D::MaxX, $Graph::GED::Graph2D::MaxY) = canvas_groesse();

    #--------------------------------------------------------------------------
    # Zeichenfläche leeren und Hintergrundfarbe setzen:
    #--------------------------------------------------------------------------
    $Konfig{canvas}->delete('all');
    $Konfig{canvas}->configure(-background => $Graph->hintergrund());
    if (length $Graph->hintergrundbild() and -f $Graph->hintergrundbild()) {
        $Konfig{canvas}->createImage(
        #    ($Graph::GED::Graph2D::MaxX-$Graph::GED::Graph2D::MinX)/2,
        #    ($Graph::GED::Graph2D::MaxY-$Graph::GED::Graph2D::MinY)/2,
        #    -anchor => 'center',
            0,
            0,
            -anchor => 'nw',
            -image  => $Konfig{mw}->Photo(-file => $Graph->hintergrundbild()),
        );
    }

    #--------------------------------------------------------------------------
    # Gitternetz einzeichnen:
    #--------------------------------------------------------------------------
    if ($Graph->gitterzeigen() or $Graph->rastimgitter()) {
        @GitterX = (); # ev. vorhandene
        @GitterY = (); # alte Einträge löschen

        for (
             my $x =  $Graph->gitterweite();
                $x <= $Graph::GED::Graph2D::MaxX;
                $x += $Graph->gitterweite()
            )
        {
            $Konfig{canvas}->createLine($x, $Graph::GED::Graph2D::MinY,
                                        $x, $Graph::GED::Graph2D::MaxY,
                                        -width => 1,
                                        -fill  => $Graph->gitterfarbe(),
                                       )
                                       if $Graph->gitterzeigen();
            push @GitterX, $x;
        }

        for (
             my $y =  $Graph->gitterweite();
                $y <= $Graph::GED::Graph2D::MaxY;
                $y += $Graph->gitterweite()
            )
        {
            $Konfig{canvas}->createLine($Graph::GED::Graph2D::MinX, $y,
                                        $Graph::GED::Graph2D::MaxX, $y,
                                        -width => 1,
                                        -fill  => $Graph->gitterfarbe(),
                                       )
                                       if $Graph->gitterzeigen();
            push @GitterY, $y;
        }
    }

    #--------------------------------------------------------------------------
    # Graphen neu einzeichnen:
    #--------------------------------------------------------------------------
    zeichne_ecke($_)  for (0..$Graph->n()-1);
    zeichne_kante($_) for (0..$Graph->m()-1);
} # sub Graph::GED::Gui::refresh


sub clear (;$) {
    #--------------------------------------------------------------------------
    # sub          : c l e a r
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Alles löschen
    # Parameter    : bind, siehe Kommentar vom 05.02.2003 (optional)
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 03.02.2003 - CD - Erstellt
    # 0.0.2 - 04.02.2003 - CD - Canvas-Widget muss nun nicht mehr übergeben
    #                           werden.
    # 0.0.3 - 05.02.2003 - CD - Parameter $bind hinzugefügt. Bei 0 wird
    #                           bind_start nicht aufgerufen, d.h. der Aufrufer
    #                           kümmert sich selbst um die Bindung. Bei ande-
    #                           ren Werten oder ohne Parameter wird bind_start
    #                           aufgerufen.
    #                           Benötigt wird dies, um das Malen von Ecken nach
    #                           Doppelklick auf Namen der Datei beim Laden zu
    #                           verhindern.
    # 0.0.4 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.5 - 09.02.2004 - CD - Umstellung auf Graphenklassen.
    # 0.0.6 - 01.01.2005 - CD - Gitter nach löschen wieder anzeigen.
    # 0.0.7 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my $bind = shift;

    $bind = 1 unless defined $bind;

    print "clear - bind = '$bind'\n" if $Konfig{debug};

    #--------------------------------------------------------------------------
    # Graphendarstellung löschen:
    #--------------------------------------------------------------------------
    $Konfig{canvas}->delete('all');

    #--------------------------------------------------------------------------
    # Graphendaten löschen:
    #--------------------------------------------------------------------------
    $Graph->clear();

    #--------------------------------------------------------------------------
    # Gitter anzeigen, falls gewünscht:
    #--------------------------------------------------------------------------
    refresh();

    #--------------------------------------------------------------------------
    # Bindung aufheben, falls man gerade am Zeichnen ist:
    #--------------------------------------------------------------------------
    $Konfig{canvas}->Tk::bind("<Motion>",  "");

    #--------------------------------------------------------------------------
    # Bindungen wieder herstellen, falls gewünscht:
    #--------------------------------------------------------------------------
    bind_start() if $bind;

} # sub Graph::GED::Gui::clear


sub zeichne_ecke ($) {
    #--------------------------------------------------------------------------
    # sub          : z e i c h n e _ e c k e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Zeichnet die angegebene Ecke ein.
    # Parameter    : Eckennummer 0 .. $Graph->n()-1
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1  - 03.02.2003 - CD - Erstellt
    # 0.0.2  - 04.02.2003 - CD - Programmabbruch bei undefinierten Koordinaten
    #                            eingebaut.
    # 0.0.3  - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                            %Graph.
    # 0.0.4  - 07.02.2004 - CD - Verwendung des Farbwertes $Graph{eckenfarbe}
    #                            statt hardvercodetem Rot.
    # 0.0.5  - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.6  - 10.02.2004 - CD - Umstellung auf die Eckenklasse.
    # 0.0.7  - 30.02.2004 - CD - Beachtung einzelner Farbwerte der Ecken.
    #                            Beachtung einzelner Größen der Ecken.
    # 0.0.8  - 01.01.2005 - CD - Titel und Gewicht mit ausgeben.
    # 0.0.9  - 03.01.2005 - CD - Anchor-Berechnung vereinfacht.
    #                            Titel- und Gewichtfarbe berücksichtigen.
    # 0.0.10 - 14.01.2005 - CD - Eckennummern auf Wunsch anzeigen.
    # 0.0.11 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my $e = shift;

    my $ecke = $Graph->ecke($e);
    my $x    = $ecke->x();
    my $y    = $ecke->y();

    die "Koordinaten von Ecke $e sind undefiniert!" unless defined $x and
                                                           defined $y;

    print "zeichne_ecke, e = '$e', (N = '",
          $Graph->n(), "')\n",
          "    (", $x, ", ", $y, ")\n",
          "    Radius = '", $Graph->radius(),
          "', Delta = '", $Graph->delta(), "'\n"
          if $Konfig{debug} > 1;

    my $farbe  = $Graph->ecke($e)->farbe()  || $Graph->eckenfarbe();
    my $radius = $Graph->ecke($e)->radius() || $Graph->radius();

    $Konfig{canvas}->
        createOval($x-$radius, $y-$radius,
                   $x+$radius, $y+$radius,
                   -width   => 1,
                   -fill    => $farbe,
                   -outline => $farbe,
                  );

    if (length $ecke->titel()) {
        my $delta = 1 + 2 * ($ecke->radius() || $Graph->radius());
        $delta = int($delta / sqrt 2) if length $ecke->titelpos() > 1;
        my $tx    = $x;
        my $ty    = $y;
        $ty -= $delta if $ecke->titelpos() =~ /N/;
        $ty += $delta if $ecke->titelpos() =~ /S/;
        $tx -= $delta if $ecke->titelpos() =~ /W/;
        $tx += $delta if $ecke->titelpos() =~ /O/;
        (my $anchor = $ecke->titelpos()) =~ tr~NWSO~senw~;
        $Konfig{canvas}->
            createText($tx, $ty,
                       -text   => $ecke->titel(),
                       -anchor => $anchor,
                       -fill   => $ecke->titelfarbe(),
                      );
    }
    if (defined $ecke->gewicht()) {
        my $delta = 1 + 2 * ($ecke->radius() || $Graph->radius());
        $delta = int($delta / sqrt 2) if length $ecke->gewichtpos() > 1;
        my $tx    = $x;
        my $ty    = $y;
        $ty -= $delta if $ecke->gewichtpos() =~ /N/;
        $ty += $delta if $ecke->gewichtpos() =~ /S/;
        $tx -= $delta if $ecke->gewichtpos() =~ /W/;
        $tx += $delta if $ecke->gewichtpos() =~ /O/;
        (my $anchor = $ecke->gewichtpos()) =~ tr~NWSO~senw~;
        $Konfig{canvas}->
            createText($tx, $ty,
                       -text   => $ecke->gewicht(),
                       -anchor => $anchor,
                       -fill   => $ecke->gewichtfarbe(),
                      );
    }
    if ($Konfig{zeige_e_nr}) {
        my $pos = 'O';
        my $delta = 1 + 2 * ($ecke->radius() || $Graph->radius());
        $delta = int($delta / sqrt 2) if length $pos > 1;
        my $tx    = $x;
        my $ty    = $y;
        $ty -= $delta if $pos =~ /N/;
        $ty += $delta if $pos =~ /S/;
        $tx -= $delta if $pos =~ /W/;
        $tx += $delta if $pos =~ /O/;
        (my $anchor = $pos) =~ tr~NWSO~senw~;
        $Konfig{canvas}->
            createText($tx, $ty,
                       -text   => $e,
                       -anchor => $anchor,
                       -fill   => $farbe,
                      );
    }
} # sub Graph::GED::Gui::zeichne_ecke


sub zeichne_kante ($) {
    #--------------------------------------------------------------------------
    # sub          : z e i c h n e _ k a n t e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Zeichnet die angegebene Kante ein.
    # Parameter    : Kantennummer 0 .. $Graph{m}-1
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1  - 03.02.2003 - CD - Erstellt
    # 0.0.2  - 04.02.2003 - CD - Programmabbruch bei undefinierten Koordinaten
    #                            eingebaut.
    # 0.0.3  - 11.02.2003 - CD - Angepasst an neue Linientypennamen.
    # 0.0.4  - 18.03.2003 - CD - Neue Linientypenkürzel verwendet.
    # 0.0.5  - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                            %Konfig.
    #                            Umstellung vieler globaler Variablen auf
    #                            %Graph.
    # 0.0.6  - 07.02.2004 - CD - Verwendung des Farbwertes $Graph{kantenfarbe}
    #                            statt hardvercodetem Schwarz.
    # 0.0.7  - 09.02.2004 - CD - Arrowshape abhängig vom Radius.
    # 0.0.8  - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.9  - 10.02.2004 - CD - Umstellung auf die Kantenklasse.
    #                            Umstellung auf die Eckenklasse.
    # 0.0.10 - 12.02.2004 - CD - Neue Hilfsmethoden x_von_a etc. verwendet.
    # 0.0.11 - 30.02.2004 - CD - Beachtung einzelner Farbwerte der Ecken.
    # 0.0.12 - 01.01.2005 - CD - Titel und Gewicht mit ausgeben.
    # 0.0.13 - 01.01.2005 - CD - Kantendicke berücksichtigen.
    #                            Kanten werden nicht mehr von den
    #                            Mittelpunkten, sondern von den Eckenrändern
    #                            aus gezogen (Koordinatenkorrektur).
    #                            Der Linienstil (Normal / Gepunkted / Ge-
    #                            strichelt) wird jetzt unterstützt.
    # 0.0.14 - 03.01.2005 - CD - Anchor-Berechnung vereinfacht.
    #                            Titel- und Gewichtfarbe berücksichtigen.
    # 0.0.15 - 14.01.2005 - CD - Kantennummern auf Wunsch anzeigen.
    # 0.0.16 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my $k = shift;

    my $kante = $Graph->kante($k);
    my $form  = $kante->form();

    die "Ecken von Kante $k sind undefiniert!" unless defined $form;

    my $xa   = $Graph->x_von_a_index($k);
    my $ya   = $Graph->y_von_a_index($k);
    my $xe   = $Graph->x_von_e_index($k);
    my $ye   = $Graph->y_von_e_index($k);

    die "Koordinaten der Anfangsecke sind undefiniert!" unless defined $xa and
                                                               defined $ya;
    die "Koordinaten der Endecke sind undefiniert!"     unless defined $xe and
                                                               defined $ye;

    print "zeichne_kante, k = '$k', (M = '",
          $Graph->m(), "')\n",
          "    ",   $kante->a(),
          " --- ",  $kante->e(),
          ",    (", $xa,
          ", ",     $ya,
          " -> ",   $xe,
          ", ",     $ye,
          ")\n"
          if $Konfig{debug} > 1;

    my $arrow = 'none';

    print "Kante $k von Ecke ", $kante->a(),
          " nach Ecke ", $kante->e(),
          ", ($xa,$ya) -> ($xe, $ye)\n"
          if $Konfig{debug} > 1;

    $arrow = 'last'  if $form eq 'k_inh';
    $arrow = 'first' if $form eq 'k_inz';
    $arrow = 'both'  if $form eq 'k_inb';

    ###########################################################################
    # Koordinatenkorrektur um die Kanten nur zwischen den Ecken zu ziehen und
    # nicht von Mittelpunkt zu Mittelpunkt:
    ###########################################################################
    my $ra   = $Graph->ecke($kante->a())->radius() || $Graph->radius();
    my $re   = $Graph->ecke($kante->e())->radius() || $Graph->radius();
    my ($xa_, $ya_, $xe_, $ye_) = ($xa, $ya, $xe, $ye);
    ($xa_, $xe_) = ($xe_, $xa_) if $xe_ < $xa_;
    ($ya_, $ye_) = ($ye_, $ya_) if $ye_ < $ya_;
    my $phi = $xe_ == $xa_ ? PI/2 : atan(($ye_-$ya_)/($xe_-$xa_));
    my $rxa = $ra * cos($phi);
    my $rya = $ra * sin($phi);
    my $rxe = $re * cos($phi);
    my $rye = $re * sin($phi);
    if ($xa == $xe) {                  # Y-Achse
        if ($ya <= $ye) {
            ($xa_, $ya_, $xe_, $ye_) = ($xa, int($ya + $rya),
                                        $xe, int($ye - $rye));
        }
        else {
            ($xa_, $ya_, $xe_, $ye_) = ($xa, int($ya - $rya),
                                        $xe, int($ye + $rye));
        }
    }
    elsif ($ya == $ye) {               # X-Achse
        if ($xa <= $xe) {
            ($xa_, $ya_, $xe_, $ye_) = (int($xa + $rxa), $ya,
                                        int($xe - $rxe), $ye);
        }
        else {
            ($xa_, $ya_, $xe_, $ye_) = (int($xa - $rxa), $ya,
                                        int($xe + $rxe), $ye);
        }
    }
    elsif ($xe >= $xa && $ye >= $ya) { # Q1
        ($xa_, $ya_, $xe_, $ye_) = (int($xa + $rxa), int($ya + $rya),
                                    int($xe - $rxe), int($ye - $rye));
    }
    elsif ($xe <= $xa && $ye <= $ya) { # Q2
        ($xa_, $ya_, $xe_, $ye_) = (int($xa - $rxa), int($ya - $rya),
                                    int($xe + $rxe), int($ye + $rye));
    }
    elsif ($xe <= $xa && $ye >= $ya) { # Q3
        ($xa_, $ya_, $xe_, $ye_) = (int($xa - $rxa), int($ya + $rya),
                                    int($xe + $rxe), int($ye - $rye));
    }
    elsif ($xe >= $xa && $ye <= $ya) { # Q4
        ($xa_, $ya_, $xe_, $ye_) = (int($xa + $rxa), int($ya - $rya),
                                    int($xe - $rxe), int($ye + $rye));
    }
    else {
        print "FEHLER in zeichne_kante()!\n";
    }
    ($xa, $ya, $xe, $ye) = ($xa_, $ya_, $xe_, $ye_);
    ###########################################################################


    my %dash = ( normal      => [1],
                 gestrichelt => '-',
                 gepunktet   => '.',
               );
    my $farbe = $kante->farbe() || $Graph->kantenfarbe();
    my $dicke = $kante->dicke() || $Graph->kantendicke();
    if ($arrow eq 'none') {
        $Konfig{canvas}->
            createLine($xa, $ya, $xe, $ye,
                       -width => $dicke,
                       -fill  => $farbe,
                       -dash  => $dash{$kante->stil()},
                      );
    }
    else {
        $Konfig{canvas}->
            createLine($xa, $ya, $xe, $ye,
                       -width      => $dicke,
                       -arrow      => $arrow,
                      #-arrowshape => [5, 7, 3],
                       -arrowshape => [
                                       2*$Graph->radius()-1 + $dicke-1,
                                       2*$Graph->radius()+1 + $dicke-1,
                                       $Graph->radius() + $dicke-1,
                                      ],
                       -fill       => $farbe,
                       -dash       => $dash{$kante->stil()},
                      );
    }


    my $x = int(($xa + $xe)/2); # Mittelpunkt der Kante
    my $y = int(($ya + $ye)/2); # berechnen...
    if (length $kante->titel()) {
        my $delta = 1 + $Graph->radius();
        $delta = int($delta / sqrt 2) if length $kante->titelpos() > 1;
        my $tx    = $x;
        my $ty    = $y;
        $ty -= $delta if $kante->titelpos() =~ /N/;
        $ty += $delta if $kante->titelpos() =~ /S/;
        $tx -= $delta if $kante->titelpos() =~ /W/;
        $tx += $delta if $kante->titelpos() =~ /O/;
        (my $anchor = $kante->titelpos()) =~ tr~NWSO~senw~;
        $Konfig{canvas}->
            createText($tx, $ty,
                       -text   => $kante->titel(),
                       -anchor => $anchor,
                       -fill   => $kante->titelfarbe(),
                      );
    }
    if (defined $kante->gewicht()) {
        my $delta = 1 + $Graph->radius();
        $delta = int($delta / sqrt 2) if length $kante->gewichtpos() > 1;
        my $tx    = $x;
        my $ty    = $y;
        $ty -= $delta if $kante->gewichtpos() =~ /N/;
        $ty += $delta if $kante->gewichtpos() =~ /S/;
        $tx -= $delta if $kante->gewichtpos() =~ /W/;
        $tx += $delta if $kante->gewichtpos() =~ /O/;
        (my $anchor = $kante->gewichtpos()) =~ tr~NWSO~senw~;
        $Konfig{canvas}->
            createText($tx, $ty,
                       -text   => $kante->gewicht(),
                       -anchor => $anchor,
                       -fill   => $kante->gewichtfarbe(),
                      );
    }
    if ($Konfig{zeige_k_nr}) {
        $Konfig{canvas}->
            createText($x, $y,
                       -text   => $k,
                       -fill   => $farbe,
                      );
    }
} # sub Graph::GED::Gui::zeichne_kante


sub lade_datei () {
    #--------------------------------------------------------------------------
    # sub          : l a d e _ d a t e i
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Ein Dialog zum Laden einer Graphendatei.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 13.02.2003 - CD - Endlich wird der Klick nach Doppelklick nicht
    #                           mehr in dem Canvas ausgewertet.
    #                           Dank Tk::after / Ishka von www.perl.de :)
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 01.01.2005 - CD - Lädt ged und ged2.
    # 0.0.5 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    # 0.0.6 - 16.01.2005 - CD - Lädt ged3 (default).
    # 0.0.7 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "lade_datei\n" if $Konfig{debug};

    #
    # Bindungen zu <Motion> und <Button-1> lösen:
    # (sonst setztz man mit dem Click auf OK ev eine Ecke, falls darunter das
    # Canvas-Widget ist.)
    #
    #my @bindings  = $Konfig{canvas}->Tk::bind("<Motion>");
    #print "Bindungen: '", @bindings, "'\n";
    #my @bindings2  = $Konfig{canvas}->Tk::bind("<Button-1>");
    #print "Bindungen2: '", @bindings2, "'\n";
    $Konfig{canvas}->Tk::bind("<Motion>",    "");
    $Konfig{canvas}->Tk::bind("<Button-1>",  "");
    #$Konfig{canvas}->Tk::bind("<Button-1>",  sub {print "JAUJAUJAU\n"});
    #@bindings  = $Konfig{canvas}->Tk::bind("<Motion>");
    #print "Bindungen: '", @bindings, "'\n";
    #@bindings2  = $Konfig{canvas}->Tk::bind("<Button-1>");
    #print "Bindungen2: '", @bindings2, "'\n";

    my $typen = [
        ['Alle Graphen Dateien',          ['.ged', '.ged2', '.ged3']],
        ['Graphen Dateien (ged3-Format)', '.ged3'                   ],
        ['Graphen Dateien (ged2-Format)', '.ged2'                   ],
        ['Graphen Dateien (ged-Format)',  '.ged'                    ],
        ['Alle Dateien',                  '*'                       ],
    ];
    my $dn = $Konfig{mw}->getOpenFile(-filetypes  => $typen,
                              -initialdir => $Konfig{pfad},
                             );

    if (defined $dn and $dn ne "") {
        $Graph->dateiname($dn);
        print "Dateiname = '$dn'\n" if $Konfig{debug};
        set_title();
        clear(0);
        Graph::GED::Data::load($dn);
        refresh();
    }

    #
    # Bindungen wieder herstellen:
    #
    #@bindings  = $Konfig{canvas}->Tk::bind("<Motion>");
    #print "Bindungen: '", @bindings, "'\n";
    #@bindings2  = $Konfig{canvas}->Tk::bind("<Button-1>");
    #print "Bindungen2: '", @bindings2, "'\n";
    $Konfig{mw}->after(1000, \&bind_start);
} # sub Graph::GED::Gui::lade_datei


sub speichere_datei () {
    #--------------------------------------------------------------------------
    # sub          : s p e i c h e r e _ d a t e i
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Ein Dialog zum Speichern einer Graphendatei.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2003 - CD - Doppelten Aufruf von save im Fall von Speichern
    #                           mit leerem Dateinamen verhindert.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    # 0.0.5 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "speichere_datei, Dateiname ist '", $Graph->dateiname(), "'\n"
          if $Konfig{debug};

    if ($Graph->dateiname() eq '') {
        speichere_datei_als();
    }
    else {
        Graph::GED::Data::save($Graph->dateiname());
    }
} # sub Graph::GED::Gui::speichere_datei


sub speichere_datei_als () {
    #--------------------------------------------------------------------------
    # sub          : s p e i c h e r e _ d a t e i _ a l s
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Ein Menü zum Speichern einer Graphendatei unter einem an-
    #                deren Namen.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 11.02.2003 - CD - -initialfile rausgenommen wegen Problemen in
    #                           tk.pm
    # 0.0.3 - 17.02.2003 - CD - Dateinamen in Historie eingetragen.
    #                           Rumpfnamen als Vorschlag (-initialfile) über-
    #                           geben.
    # 0.0.4 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.5 - 01.01.2005 - CD - Speichert in den Formaten ged und ged2
    #                           (default).
    # 0.0.6 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    # 0.0.7 - 16.01.2005 - CD - Neues Format ged3 ist default.
    # 0.0.8 - 17.01.2005 - CD - Punkt in Dateinamen einfügen, falls er fehlt
    #                           (dies ist unter Linux der Fall).
    # 0.0.9 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "speichere_datei_als, Dateiname ist '", $Graph->dateiname(), "'\n"
        if $Konfig{debug};

    my $name = $Graph->dateiname();
    $name =~ s~.*/([^/]*)\.ged[23]?$~$1~g; #~

    my $typen = [
        ['Graphen Dateien (ged3-Format)', '.ged3'],
        ['Graphen Dateien (ged2-Format)', '.ged2'],
        ['Graphen Dateien (ged-Format)',  '.ged' ],
        ['Alle Dateien',                  '*'    ],
    ];
    my $dn = $Konfig{mw}->getSaveFile(-filetypes        => $typen,
                              -defaultextension => '.ged3',
                              -initialfile      => $name,
#                              -initialdir       => $Konfig{pfad},
# Entfällt, da die Datei mit absolutem Pfad angegeben wird...
                             );

    if (defined $dn and $dn ne "") {
        #if ($dateiname =~ m~[^.]ged\d?$~) {
        #    $dateiname =~ s~([^.])(ged\d?$)~$1.$2~;
        #}
        $Graph->dateiname($dn);
        print "Dateiname = '$dn'\n" if $Konfig{debug};
        set_title();
        speichere_datei();
        Graph::GED::Data::add($dn);
    }
} # sub Graph::GED::Gui::speichere_datei_als


sub lade_animation () {
    #--------------------------------------------------------------------------
    # sub          : l a d e _ a n i m a t i o n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Ein Dialog zum Laden einer Graphendatei.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 19.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "lade_animation\n" if $Konfig{debug};

    #
    # Bindungen zu <Motion> und <Button-1> lösen:
    # (sonst setztz man mit dem Click auf OK ev eine Ecke, falls darunter das
    # Canvas-Widget ist.)
    #
    $Konfig{canvas}->Tk::bind("<Motion>",    "");
    $Konfig{canvas}->Tk::bind("<Button-1>",  "");

    my $typen = [
       #['Alle Animations Dateien',             ['.gedanim',] ],
        ['Animations Dateien (gedanim-Format)', '.gedanim'    ],
        ['Alle Dateien',                        '*'           ],
    ];
    my $dn = $Konfig{mw}->getOpenFile(
        -filetypes  => $typen,
        -initialdir => $Konfig{pfad},
    );

    if (defined $dn and $dn ne '') {
        $Konfig{mw}->Busy(-recurse => 1);
        if ($Animation->load($dn)) {
            $Animation->filename($dn);
            print "Animations-Dateiname = '$dn'\n" if $Konfig{debug};
            set_title();
            $Konfig{statuszeile} = "Animation '$dn' geladen.";
            $Graph   = $Animation->graph(0);
            $History = $Animation->history(0);
            $Konfig{animationskala}->set(1);
            $Konfig{animationskala}->configure(
                -to => $Animation->number_of_graphs()
            );
            refresh();
        }
        $Konfig{mw}->Unbusy();
    }

    #
    # Bindungen wieder herstellen:
    #
    $Konfig{mw}->after(1000, \&bind_start);

} # sub Graph::GED::Gui::lade_animation


sub speichere_animation () {
    #--------------------------------------------------------------------------
    # sub          : s p e i c h e r e _ a n i m a t i o n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Ein Dialog zum Speichern einer Graphendatei.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 19.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "speichere_animation, Dateiname ist '", $Animation->filename(),
          "'\n" if $Konfig{debug};

    if ($Animation->filename() eq '') {
        speichere_animation_als();
    }
    else {
        $Konfig{mw}->Busy(-recurse => 1);
        if ($Animation->save($Animation->filename())) {
            $Konfig{statuszeile} = "Animation '" . $Animation->filename() .
                                   "' gespeichert.";
        }
        $Konfig{mw}->Unbusy();
    }
} # sub Graph::GED::Gui::speichere_animation


sub speichere_animation_als () {
    #--------------------------------------------------------------------------
    # sub          : s p e i c h e r e _ a n i m a t i o n _ a l s
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Ein Menü zum Speichern der Animation unter einem anderen
    #                Namen.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 19.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "speichere_animation_als, Dateiname ist '", $Animation->filename(),
          "'\n" if $Konfig{debug};

    my $name = $Animation->filename();
    $name =~ s~.*/([^/]*)\.ged[23]?anim$~$1~g; #~

    my $typen = [
       #['Alle Animations Dateien',             ['.gedanim',] ],
        ['Animations Dateien (gedanim-Format)', '.gedanim'    ],
        ['Alle Dateien',                        '*'           ],
    ];
    my $dn = $Konfig{mw}->getSaveFile(
        -filetypes        => $typen,
        -defaultextension => '.gedanim',
        -initialfile      => $name,
    );

    if (defined $dn and $dn ne "") {
        $Animation->filename($dn);
        print "Animation-Dateiname = '$dn'\n" if $Konfig{debug};
        set_title();
        speichere_animation();
    }
} # sub Graph::GED::Gui::speichere_animation_als


sub lade_hintergrundbild () {
    #--------------------------------------------------------------------------
    # sub          : l a d e _ h i n t e r g r u n d b i l d
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Ein Dialog zum Laden des Hintergrundbildes.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 14.01.2005 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "lade_hintergrundbild\n" if $Konfig{debug};

    #
    # Bindungen zu <Motion> und <Button-1> lösen:
    # (sonst setztz man mit dem Click auf OK ev eine Ecke, falls darunter das
    # Canvas-Widget ist.)
    #
    $Konfig{canvas}->Tk::bind("<Motion>",    "");
    $Konfig{canvas}->Tk::bind("<Button-1>",  "");

    my $typen = [
        ['GIF- und JPG-Dateien',   ['gif', '.jpg', '.jpeg']],
        ['JPG-Dateien',            ['.jpg', '.jpeg']       ],
        ['GIF-Dateien',            '.ged'                  ],
        ['Alle Dateien',           '*'                     ],
    ];
    my $dn = $Konfig{mw}->getOpenFile(-filetypes  => $typen,
                              -initialdir => $Konfig{pfad},
                             );

    if (defined $dn and $dn ne "") {
        Graph::GED::Logic::aenderung();
        $Graph->hintergrundbild($dn);
        refresh();
    }

    #
    # Bindungen wieder herstellen:
    #
    $Konfig{mw}->after(1000, \&bind_start);

} # sub Graph::GED::Gui::lade_hintergrundbild


sub export_nach_latex () {
    #--------------------------------------------------------------------------
    # sub          : e x p o r t _ n a c h _ l a t e x
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Ein Menü zum Speichern einer Graphendatei im LaTeX-Format.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 13.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.3 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    #                           toLaTeX nach Graph2D verschoben.
    # 0.0.4 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "export_nach_latex, Dateiname ist '", $Graph->dateiname(), "'\n"
        if $Konfig{debug};

    ( my $name = $Graph->dateiname() ) =~ s/\.ged$/.tex/;

    my $typen = [
        ['LaTeX-Dateien',           '.tex'],
        ['Interne Graphen Dateien', '.ged'],
        ['Alle Dateien',            '*'   ],
    ];
    my $dn = $Konfig{mw}->
                getSaveFile(-title            => "Graph nach LaTeX exportieren",
                            -filetypes        => $typen,
                            -defaultextension => 'tex',
#                             -initialfile      => $name,
# macht leider auch Probleme
#                             -initialfile      => $Graph->dateiname(),
# Dann knallts in tk.pm ...
#                             -initialdir       => $Konfig{pfad},
# Entfällt, da die Datei mit absolutem Pfad angegeben wird...
                             );

    if (defined $dn and $dn ne "") {
        print "LaTeX-Dateiname = '$dn'\n" if $Konfig{debug} > 1;
        $Graph->toLaTeX($dn);
        $Konfig{statuszeile} = "Datei als '$name' nach LaTeX exportiert.";
    }

} # sub Graph::GED::Gui::export_nach_latex


sub export_nach_postscript () {
    #--------------------------------------------------------------------------
    # sub          : e x p o r t _ n a c h _ p o s t s c r i p t
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Ein Menü zum Speichern einer Graphendatei im PS-Format.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 13.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.3 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    # 0.0.4 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "export_nach_postscript, Dateiname ist '", $Graph->dateiname(), "'\n"
        if $Konfig{debug};

    ( my $name = $Graph->dateiname() ) =~ s/\.ged$/.ps/;

    my $typen = [
        ['Postscript-Dateien',      '.ps'],
        ['Interne Graphen Dateien', '.ged'],
        ['Alle Dateien',            '*'   ],
    ];
    my $dn = $Konfig{mw}->
                getSaveFile(-title            => "Graph nach Postscript " .
                                                 "exportieren",
                            -filetypes        => $typen,
                            -defaultextension => 'ps',
#                            -initialfile      => $name,
# macht leider auch Probleme
#                            -initialfile      => $Graph->dateiname(),
# Dann knallts in tk.pm ...
#                            -initialdir       => $Konfig{pfad},
# Entfällt, da die Datei mit absolutem Pfad angegeben wird...
                             );

    if (defined $dn and $dn ne "") {
        print "Postscript-Dateiname = '$dn'\n" if $Konfig{debug} > 1;
        $Konfig{canvas}->postscript(-file => $dn);

        $Konfig{statuszeile} = "Datei als '$name' nach Postscript exportiert.";
    }
} # sub Graph::GED::Gui::export_nach_postscript


sub dump_eingeben () {
    #--------------------------------------------------------------------------
    # sub          : d u m p _ e i n g e b e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Ermöglicht es, eine interne Datei / einen Dump per Hand
    #                einzugeben.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 12.03.2003 - CD - Funktionalität eingebaut
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.4 - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.5 - 12.02.2004 - CD - aenderung() vor Änderungen aufgerufen.
    # 0.0.6 - 16.01.2005 - CD - Dateiname liegt jetzt in $Graph.
    # 0.0.7 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "dump_eingeben\n" if $Konfig{debug};

    sub load_dump {
        print "load_dump\n" if $Konfig{debug};

        my $text = shift;

        $Graph->clear();

        # Linientypenhash aufbauen:
        my %typen;
        for (keys %Linientypen) {
            $typen{$Linientypen{$_}} = $_;
        }


        my @lines = split /\n/, $text;

        my $hx = 0;
        my $hy = 0;
        my $he = 0;
        my $ha = 0;
        my $hf = 0;

        my $l = ''; # aktuelle Zeile
        my $z = 0;  # Zeilennummer

        # Kommentarkopf überlesen:
        do {
            $l = shift @lines;
            ++$z;
        } while '#' eq substr $l, 0, 1;

        # Eckenzahl einlesen:
        return "Fehler in Zeile $z - N fehlt!" unless defined $l;
        (my $n) = $l =~ /^\s*(\d+)\s*$/;
        return "Fehler in Zeile $z - N fehlt!" unless defined $1;


        # Eckenkoordinaten einlesen:
        for (0..$n-1) {
            $l = shift @lines;
            ++$z;
            return "Fehler in Zeile $z - Eckenkoordinate fehlt!"
                unless defined $l;
            $l =~ /^\s*(\d+)\s*;\s*(\d+)\s*$/;
            return "Fehler in Zeile $z - Eckenkoordinate fehlt!"
                unless defined $1 and defined $2;
            $Graph->ecke_einfuegen($1, $2);
        }


        # Kantenzahl einlesen:
        $l = shift @lines;
        ++$z;
        return "Fehler in Zeile $z - M fehlt!" unless defined $l;
        chomp $l;
        (my $m) = $l =~ /^\s*(\d+)\s*$/;

        return "Fehler in Zeile $z - M fehlt!" unless defined $1;

        # Eckenkoordinaten einlesen:
        for (0..$m-1) {
            $l = shift @lines;
            ++$z;
            return "Fehler in Zeile $z - Kantendaten fehlen!"
                unless defined $l;
            $l =~ /^\s*(\d+)\s*;\s*(\d+)\s*;\s*(.*)\s*$/;
            return "Fehler in Zeile $z - Kantendaten fehlen!"
                unless defined $1 and defined $2 and defined $3;
            $Graph->kante_einfuegen($1, $2, exists $typen{$3}?$typen{$3}:$3);
        }

        return '';
    } # sub Graph::GED::Gui::load_dump


    #
    # Neues Toplevel-Fenster zur Texteingabe erzeugen:
    #
    my $tl = $Konfig{mw}->Toplevel();
    $tl->title('Interne Daten');
    $tl->Label(
        -text   => 'Interne Datenstruktur im GED-Format:',
    )->pack(
        -anchor => 'w',
    );
    my $txt = $tl->Scrolled("Text",
                        -scrollbars => 'osoe',
                        -background => '#d7d7d7',
                        -foreground => 'blue',
                        -width      => 40,
                        -height     => 25,
                        -relief     => 'flat',
                        -font       => '{Courier New} 10 {normal}',
                        -wrap       => 'none',
                       )
                ->pack();
    $tl->Button(-text    => 'Dump laden',
                -command => sub {Graph::GED::Logic::aenderung();
                                 Graph::GED::Data::neue_datei();
                                 my $rtext = load_dump($txt->get('1.0', 'end'));
                                 if ($rtext eq '') {
                                     $Graph->dateiname('');
                                     $Konfig{mw}->title('G - E - D   ' . $Konfig{version});
                                     $Konfig{statuszeile} = 'Dump erfolgreich eingelesen.';
                                     refresh();
                                     $tl->destroy();
                                 }
                                 else {
                                     print "\a";
                                     $Konfig{statuszeile} = $rtext;
                                 }
                                },
               )
         ->pack(-expand => '1',
                -fill   => 'x',
                -side   => 'left',
               );


    #
    # Rest: grab verhindert das Zurückschalten auf $WM per Klick, focus sorgt
    #       dafür, dass $tl und in diesem $txt auch wirklich aktiv ist:
    #
    $tl->grab();
    $txt->focus();
} # sub Graph::GED::Gui::dump_eingeben


sub zeige_daten_an ($) {
    #--------------------------------------------------------------------------
    # sub          : z e i g e _ d a t e n _ a n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Zeigt die interne Datenstruktur in einem Fenster an.
    # Parameter    : Format ('.ged', '.ged2' ...)
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    #                           Umstellung vieler globaler Variablen auf
    #                           %Graph.
    # 0.0.3 - 08.02.2004 - CD - anzeige_fenster() zum Anzeigen verwendet.
    # 0.0.4 - 09.02.2004 - CD - Umstellungen auf die Verwendung von Graph2D.
    # 0.0.5 - 10.02.2004 - CD - Umstellung auf die Kantenklasse.
    #                           Umstellung auf die Eckenklasse.
    # 0.0.6 - 12.02.2004 - CD - anzeige_fenster() nun aus Graph::GED::TkTools
    #                           aufgerufen.
    # 0.0.7 - 16.01.2005 - CD - Die Daten werden jetzt in eine temporäre Datei
    #                           gespeichert.
    # 0.0.8 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my ($w, $format) = @_;
    $format = $w unless defined $format; # Mal aufruf via bind mal direkt.

    print "zeige_daten_an (Format '$format')\n" if $Konfig{debug};

    #
    # Temporären Dateinamen ermitteln:
    # 
    my ($fh, $fn) = File::Temp::tempfile('tempXXXXX',
                                         SUFFIX => $format,
                                         DIR    => "$FindBin::RealBin/data",
                                        );
    close $fh; # Sonst funktioniert das unlink nicht.

    #
    # Graphen unter diesem Namen speichern:
    # 
    my $changed  = $Konfig{changed};
    my $dateiname = $Graph->dateiname();

    $Graph->save($fn);
    $Konfig{changed} = $changed;
    $Graph->dateiname($dateiname);

    #
    # Geschriebene Datei einlesen:
    #
    open (F, $fn) or die $!;
    my $text;
    {local $/; $text = <F>;}
    close F or warn $!;

    #
    # Temporäre Datei löschen:
    # 
    unlink $fn;

    #
    # Text ausgeben:
    #
    Graph::GED::TkTools::anzeige_fenster($Konfig{mw},
                                         "Interne Daten im '$format'-Format",
                                         'Interne Datenstruktur:',
                                         '{Courier New} 10 {normal}',
                                         $text,
                                         0,
                                        );
} # sub Graph::GED::Gui::zeige_daten_an


sub zeige_daten_an_dumper () {
    #--------------------------------------------------------------------------
    # sub          : z e i g e _ d a t e n _ a n _ d u m p e r
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Zeigt die interne Datenstruktur in einem Fenster an.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 31.12.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "zeige_daten_an_dumper\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster($Konfig{mw},
                                         'Interne Daten (Dumper)',
                                         'Interne Datenstruktur gedumpt:',
                                         '{Courier New} 10 {normal}',
                                         Dumper($Graph),
                                         0,
                                        );
} # sub Graph::GED::Gui::zeige_daten_an_dumper


sub hilfe () {
    #--------------------------------------------------------------------------
    # sub          : h i l f e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Zeigt einen Hilfstext an.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2003 - CD - Neues Toplevel-Fenster zur Anzeige spendiert.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 08.02.2004 - CD - anzeige_fenster() zum Anzeigen verwendet.
    # 0.0.5 - 12.02.2004 - CD - anzeige_fenster() nun aus Graph::GED::TkTools
    #                           aufgerufen.
    # 0.0.6 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "hilfe\n" if $Konfig{debug};

    my $text = <<"HILFETEXT";
     GED   V$Konfig{version} vom $Konfig{version_datum}
     ~~~

...
HILFETEXT

    Graph::GED::TkTools::anzeige_fenster($Konfig{mw},
                                         'Hilfe zu GED',
                                         'Hilfe:',
                                         '{Courier New} 10 {normal}',
                                         $text,
                                         0,
                                        );
} # sub Graph::GED::Gui::hilfe


sub graph_erkl () {
    #--------------------------------------------------------------------------
    # sub          : g r a p h _ e r k l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Gibt eine Erklärung zu Graphen aus.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2003 - CD - Neues Toplevel-Fenster zur Anzeige spendiert.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 08.02.2004 - CD - anzeige_fenster() zum Anzeigen verwendet.
    # 0.0.5 - 12.02.2004 - CD - anzeige_fenster() nun aus Graph::GED::TkTools
    #                           aufgerufen.
    # 0.0.6 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "graph_erkl\n" if $Konfig{debug};

    my $text = <<'GRAPHERKL';
G - E - D ist ein grafischer Editor für Graphen aus der Graphentheorie.

Graphen bestehen aus einer Menge von Ecken N = {e_1, e_2, ..., e_n} und einer
Menge von Kanten M = {k_1, k_2, ..., k_m}, welche jeweils zwei Ecken
miteinander verbinden.

Die Kanten kann man sich deshalb jeweils als ein Paar von Ecken vorstellen,
wenn die Kante k1 die Ecke e1 mit der Ecke e3 verbindet, kann man schreiben:
k1 = (e1, e3).
GRAPHERKL

    Graph::GED::TkTools::anzeige_fenster($Konfig{mw},
                                         'Was sind Graphen?',
                                         'Was sind Graphen?',
                                         '{Courier New} 10 {normal}',
                                         $text,
                                         0,
                                        );
} # sub Graph::GED::Gui::graph_erkl


sub history () {
    #--------------------------------------------------------------------------
    # sub          : h i s t o r y
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Zeigt die History an.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 13.02.2003 - CD - Erstellt
    # 0.0.2 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.3 - 08.02.2004 - CD - anzeige_fenster() zum Anzeigen verwendet.
    # 0.0.4 - 12.02.2004 - CD - anzeige_fenster() nun aus Graph::GED::TkTools
    #                           aufgerufen.
    # 0.0.5 - 18.01.2005 - CD - Text nach Graph::GED::Text ausgelagert.
    # 0.0.6 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "history\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster($Konfig{mw},
                                         'History',
                                         'History:',
                                         '{Courier New} 10 {normal}',
                                         $Graph::GED::Text::Historytext,
                                         0,
                                        );
} # sub Graph::GED::Gui::history


sub todo () {
    #--------------------------------------------------------------------------
    # sub          : t o d o
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : ToDo-Text anzeigen
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2003 - CD - Erstellt
    # 0.0.2 - 13.02.2003 - CD - Text in globale Variable verschoben, damit die
    #                           ToDo-Liste wieder am Anfang dieser Datei steht.
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 08.02.2004 - CD - anzeige_fenster() zum Anzeigen verwendet.
    # 0.0.5 - 12.02.2004 - CD - anzeige_fenster() nun aus Graph::GED::TkTools
    #                           aufgerufen.
    # 0.0.6 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "todo\n" if $Konfig{debug};

    Graph::GED::TkTools::anzeige_fenster($Konfig{mw},
                                         'Ideen',
                                         'Ideen:',
                                         '{Courier New} 10 {normal}',
                                         $Graph::GED::Text::Todotext,
                                         0,
                                        );
} # sub Graph::GED::Gui::todo


sub show_perldoc () {
    #--------------------------------------------------------------------------
    # sub          : s h o w _ p e r l d o c
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Perldoc-Text anzeigen
    # Parameter    : Datei
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 18.01.2004 - CD - Erstellt
    #--------------------------------------------------------------------------

    print "todo\n" if $Konfig{debug};

    my ($f) = @_;

    Graph::GED::TkTools::anzeige_fenster(
        $Konfig{mw},
        'perldoc ged.pl',
        'perldoc ged.pl:',
        '{Courier New} 10 {normal}',
        qx(perldoc $FindBin::RealBin/$f),
        0,
    );
} # sub Graph::GED::Gui::perldoc_graph_ged_basis_pm
sub ueber_ged () {
    #--------------------------------------------------------------------------
    # sub          : u e b e r _ g e d
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Zeigt etwas Informationen über GED an.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 04.02.2003 - CD - Erstellt
    # 0.0.2 - 10.02.2003 - CD - GED-Icon verwendet
    # 0.0.3 - 05.02.2004 - CD - Umstellung vieler globaler Variablen auf
    #                           %Konfig.
    # 0.0.4 - 09.02.2004 - CD - Hinweis auf die Webseite eingebaut.
    # 0.0.5 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "ueber_ged\n" if $Konfig{debug};

    my $icon = $Konfig{mw}->Photo(-file   => "$FindBin::RealBin/res/ged.bmp",
                          -height => 32,
                          -width  => 32,
                         );
    my $dialog = $Konfig{mw}->Dialog(-text           => <<'HERE2' . "Version $Konfig{version} / $Konfig{version_datum}",
G - E - D ist ein grafischer Editor für Graphen aus der Graphentheorie.

GED bietet die Möglichkeit, Ecken und Kanten leicht mit der Maus auf der
Arbeitsfläche zu positionieren und erlaubt es, den entstandenen Graphen in
unterschiedlichen Formaten abzuspeichern.

Autor: Christian Dühl (e.mail: christian@duehl.de)

Webseite: http://www.duehl.de/christian/perl/ged.html

HERE2
                                #-bitmap         => 'info',
                                 -image          => $icon,
                                 -title          => 'Über G-E-D',
                                 -default_button => 'OK',
                                 -buttons        => [ 'OK' ],
                                #-font           => '{Courier New} 10 {normal}',
                                 -font           => '{Times New Roman} 12 {normal}',
                                );
    $dialog->Show(-global);
} # sub Graph::GED::Gui::ueber_ged


sub canvas_groesse () {
    #--------------------------------------------------------------------------
    # sub          : c a n v a s _ g r o e s s e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Ermittelt die Koordinaten des sichtbaren Bereiches der
    #                Zeichenfläche.
    # Parameter    : keine
    # Rückgabewert : (xmin, ymin, xmax, ymax)
    #--------------------------------------------------------------------------
    # 0.0.1 - 06.02.2004 - CD - Erstellt
    # 0.0.2 - 07.02.2004 - CD - Geometry zur Größenermittlung verwendet.
    # 0.0.3 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    print "canvas_groesse\n" if $Konfig{debug};


    #my ($x, $y, $w, $h) = $Konfig{canvas}->bbox("all");
    #print "x=$x y=$y w=$w h=$h\n";
    #use Data::Dumper;
    #print Dumper($Konfig{canvas}->bbox("all")), "\n";

    my $geom           = $Konfig{canvas}->geometry();
    my ($wh, $px, $py) = split /\+/, $geom;
    my ($xx, $yy)      = split /x/, $wh;
    print "$geom ($xx x $yy + $px + $py)\n" if $Konfig{debug};


    return (0, 0, $xx, $yy);   #(0,0, 486, 430);
} # sub Graph::GED::Gui::canvas_groesse


sub waehle_eckenfarbe () {
    #--------------------------------------------------------------------------
    # sub          : w a e h l e _ e c k e n f a r b e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Wählt die Eckenfarbe per Dialog.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 09.02.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my $farbe = $Konfig{mw}->chooseColor(-title        => 'Eckenfarbe wählen',
                                         -initialcolor => $Graph->eckenfarbe(),
                                        );
    $Graph->eckenfarbe($farbe) if $farbe;
    refresh();
} # sub Graph::GED::Gui::waehle_eckenfarbe


sub waehle_kantenfarbe () {
    #--------------------------------------------------------------------------
    # sub          : w a e h l e _ k a n t e n f a r b e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Wählt die Kantenfarbe per Dialog.
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 09.02.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my $farbe = $Konfig{mw}->chooseColor(-title        => 'Kantenfarbe wählen',
                                         -initialcolor => $Graph->kantenfarbe(),
                                        );
    $Graph->kantenfarbe($farbe) if $farbe;
    refresh();
} # sub Graph::GED::Gui::waehle_kantenfarbe


sub historie_schalter_aktualisieren ($$) {
    #--------------------------------------------------------------------------
    # sub          : h i s t o r i e _ s c h a l t e r
    #                                             _ a k t u a l i s i e r e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Aktiviert oder deaktiviert die Rückgängig-/Wiederholen-
    #                Schalter.
    # Parameter    : 1) Anzahl Rückgängigmöglichkeiten
    #                2) Anzahl Wiederholungsmöglichkeiten
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 12.02.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my $alt = shift;
    my $neu = shift;


    print "historie_schalter_aktualisieren($alt,$neu)\n" if $Konfig{debug};

    if ($alt) {
        $Konfig{rueck_btn}->configure(-state => 'normal');
    }
    else {
        $Konfig{rueck_btn}->configure(-state => 'disabled');
    }
    if ($neu) {
        $Konfig{wieder_btn}->configure(-state => 'normal');
    }
    else {
        $Konfig{wieder_btn}->configure(-state => 'disabled');
    }
} # sub Graph::GED::Gui::historie_schalter_aktualisieren


sub context_menu ($$$$$) {
    #--------------------------------------------------------------------------
    # sub          : c o n t e x t _ m e n u
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Kontextmenue anzeigen.
    # Parameter    : 1) Canvas Widget
    #                2) x-Koordinate
    #                3) y-Koordinate
    #                4) absolute x-Koordinate
    #                5) absolute y-Koordinate
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    # 0.0.2 - 31.12.2004 - CD - Ausrichtungen für Wert und Titel.
    # 0.0.3 - 01.01.2005 - CD - Ecke mergen, löschen, am Gitter ausrichten
    #                           Kante löschen.
    # 0.0.4 - 01.01.2005 - CD - Kantendicke.
    # 0.0.4 - 02.01.2005 - CD - tk_cm_farbwahl_klein() durch
    #                           Graph::GED::TkTools::farbwahl() ersetzt.
    #                           Ecken- und Kantenfarbe auch per kleiner
    #                           Farbauswahl wählbar.
    #                           Kontextmenü abseits von Ecken und Kanten
    #                           eingeführt (mit Wahl der Hintergrundfarbe).
    # 0.0.5 - 03.01.2005 - CD - Gitterdaten liegen nun in $Graph.
    #                           Die Gittereinstellungen sind nun auch per
    #                           Kontextmenü möglich.
    #                           Selbiges gilt für die Defaultecken- und
    #                           kantenfarbe.
    # 0.0.6 - 14.01.2005 - CD - Hintergrundbild laden.
    # 0.0.7 - 16.01.2005 - CD - Default Kantendicke bestimmen.
    # 0.0.8 - 17.01.2005 - CD - Für Probleme unter Linux: Überflüssige
    #                           Kontextmenüs zerstören.
    # 0.0.9 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my ($canvas, $x, $y, $ax, $ay) = @_;
    print "context-Menue ($x, $y) - ($ax, $ay)\n" if $Konfig{debug};


    #--------------------------------------------------------------------------
    # Herausfinden, welches Element gemeint ist:
    #--------------------------------------------------------------------------
    my $ek = 0;                            # 0 nix, 1 Ecke, 2 Kante
    my $nr = $Graph->meint_ecke($x, $y);   # Nummer von Ecke oder Kante

    if ($nr > -1) {
        $ek = 1;
        #print "meint Ecke $nr\n";
    }
    else {
        $nr = $Graph->meint_kante($x, $y);
        if ($nr > -1) {
            #print "meint Kante $nr\n";
            $ek = 2;
        }
        else {
            $ek = 0;
            #print "Kein Element getroffen!\n";
        }
    }
    
    clear_context_menu();
    my $cmenu;
    if ($ek == 0) {
        $cmenu =
        $Konfig{mw}->Menu(
            -font      => $Konfig{menufont},
            -tearoff   => 0,
            -menuitems => [
                ["command"    => "Hintergrundfarbe ändern (kleine Farbauswahl)",
                 -command     => [ \&Graph::GED::TkTools::farbwahl,
                                   $Konfig{mw},
                                   $ax, $ay,
                                   sub {
                                       Graph::GED::Logic::aenderung();
                                       $Graph->hintergrund($_[0]);
                                       refresh();
                                   },
                                 ],
                 -font        => $Konfig{menufont},
                ],
                ["command"    => "Hintergrundfarbe ändern (große Farbauswahl)",
                 -command     => sub {
                                     my $c = $Konfig{mw}->chooseColor(-title
                                         => "Bitte die Farbe für den "
                                            . "Hintergrund auswählen:");
                                        Graph::GED::Logic::aenderung();
                                     $Graph->hintergrund($c) if $c;
                                     refresh();
                                 },
                 -font        => $Konfig{menufont},
                ],
                ["command"    => "Hintergrundbild festlegen",
                 -command     => \&lade_hintergrundbild,
                 -font        => $Konfig{menufont},
                ],
                '-',
                ['checkbutton' => "Eckennummern anzeigen",
                 -variable     => \$Konfig{zeige_e_nr},
                 -command      => \&refresh,
                 -font         => $Konfig{menufont},
                ],
                ['checkbutton' => "Kantennummern anzeigen",
                 -variable     => \$Konfig{zeige_k_nr},
                 -command      => \&refresh,
                 -font         => $Konfig{menufont},
                 ],
                '-',
                ['checkbutton' => "Gitternetz anzeigen",
                 -variable     => \$Graph->{gitterzeigen}, #!!!
                 -command      => \&refresh,
                 -underline    => 0,
                 -font         => $Konfig{menufont},
                ],
                [qw(cascade Gitterweite -tearoff 1 -underline 6
                    -menuitems),
                 [map ['radiobutton' => $_,
                       -variable => \$Graph->{gitterweite}, #!!!
                       -command  => \&refresh,
                      ],
                      qw(5 10 15 20 25 30 35 40 45 50
                         60 70 80 100),
                 ],
                ],
                ['checkbutton' => "Am Gitternetz einrasten",
                 -variable     => \$Graph->{rastimgitter}, #!!!
                 -command      => \&refresh,
                 -underline    => 17,
                 -font         => $Konfig{menufont},
                ],
                ['command'     => "Gitterfarbe wählen (kleine Auswahl)",
                 -command      => sub {
                                      my ($x, $y) = $Konfig{mw}->geometry()
                                          =~ /\+(\d+)\+(\d+)$/;
                                      Graph::GED::TkTools::farbwahl(
                                          $Konfig{mw},
                                          $x+140, $y+155,
                                          sub {
                                              Graph::GED::Logic::aenderung();
                                              $Graph->gitterfarbe($_[0]);
                                              refresh();
                                          },
                                      )
                                  },
                 -font         => $Konfig{menufont},
                 -underline    => 0,
                ],
                ['command'     => "Gitterfarbe wählen (große Auswahl)",
                 -command      => sub {
                     Graph::GED::Logic::aenderung();
                                      my $farbe = $Konfig{mw}->
                                          chooseColor(-title        => 'Gitterfarbe wählen',
                                                      -initialcolor => $Graph->hintergrund(),
                                                     );
                                      $Graph->gitterfarbe($farbe) if $farbe;
                                      refresh();
                                  },
                 -font         => $Konfig{menufont},
                 -underline    => 0,
                ],
                '-',
                ['command'     => "Default Eckenfarbe wählen (kleine Auswahl)",
                 -command      => sub {
                                      my ($x, $y) = $Konfig{mw}->geometry()
                                          =~ /\+(\d+)\+(\d+)$/;
                                      Graph::GED::TkTools::farbwahl(
                                          $Konfig{mw},
                                          $x+140, $y+165,
                                          sub {
                                              Graph::GED::Logic::aenderung();
                                              $Graph->eckenfarbe($_[0]);
                                              refresh();
                                          },
                                      )
                                  },
                 -font         => $Konfig{menufont},
                 -underline    => 0,
                ],
                ['command'     => "Default Kantenfarbe wählen (kleine Auswahl)",
                 -command      => sub {
                                      my ($x, $y) = $Konfig{mw}->geometry()
                                          =~ /\+(\d+)\+(\d+)$/;
                                      Graph::GED::TkTools::farbwahl(
                                          $Konfig{mw},
                                          $x+140, $y+175,
                                          sub {
                                              Graph::GED::Logic::aenderung();
                                              $Graph->kantenfarbe($_[0]);
                                              refresh();
                                          },
                                      )
                                  },
                 -font         => $Konfig{menufont},
                 -underline    => 0,
                ],
                '-',
                ['command'     => "Default Eckenfarbe wählen (große Farbauswahl)",
                 -command      => \&waehle_eckenfarbe,
                 -font         => $Konfig{menufont},
                 -underline    => 0,
                ],
                ['command'     => "Default Kantenfarbe wählen (große Farbauswahl)",
                 -command      => \&waehle_kantenfarbe,
                 -font         => $Konfig{menufont},
                 -underline    => 0,
                ],
                '-',
                ["command"    => "Default Eckenradius ändern",
                 -command     => [ \&tk_cm_groessenwahl, $ax, $ay, $Graph ],
                 -font        => $Konfig{menufont},
                ],
                ["command"    => "Default Kantendicke ändern",
                 -command     => [ \&tk_cm_kantendicke, $ax, $ay, $Graph ] ,
                 -font        => $Konfig{menufont},
                ],
                          ],
        );
    }
    else {
        #print +($ek == 1 ? 'Ecke' : 'Kante'), " $nr ist gemeint\n";

        my $element = ($ek == 1 ? 'Ecke' : 'Kante');
        my $objekt  = ($ek == 1 ? $Graph->ecke($nr) : $Graph->kante($nr));

        $cmenu =
        $Konfig{mw}->Menu(-font      => $Konfig{menufont},
                          -tearoff   => 0,
                          -menuitems => [
                              ["command"    => "${element}nfarbe ändern (kleine Farbauswahl)",
                               -command     => [ \&Graph::GED::TkTools::farbwahl,
                                                 $Konfig{mw},
                                                 $ax, $ay,
                                                 sub {
                                                     Graph::GED::Logic::aenderung();
                                                     $objekt->farbe($_[0]);
                                                     refresh();
                                                 },
                                               ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nfarbe ändern (große Farbauswahl)",
                               -command     => sub {
                                                   my $c = $Konfig{mw}->chooseColor(-title
                                                       => "Bitte die Farbe für $element $nr"
                                                          . " auswählen:");
                                                   $objekt->farbe($c) if $c;
                                                   refresh();
                                               },
                               -font        => $Konfig{menufont},
                              ],
                              '-',
                              ($ek == 1
                              ? # nur für Ecken:
                              ["command"    => "${element}ngröße ändern",
                               -command     => [ \&tk_cm_groessenwahl, $ax, $ay, $objekt ],
                               -font        => $Konfig{menufont},
                              ]
                              : # nur für Kanten:
                              (
                              ["command"    => "${element}nform ändern",
                               -command     => [ \&tk_cm_kantenformwahl, $ax, $ay, $objekt ] ,
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nstil ändern",
                               -command     => [ \&tk_cm_kantenstilwahl, $ax, $ay, $objekt ] ,
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}ndicke ändern",
                               -command     => [ \&tk_cm_kantendicke, $ax, $ay, $objekt ] ,
                               -font        => $Konfig{menufont},
                              ])),
                              '-',
                              ["command"    => "${element}nbeschriftung ändern",
                               -command     => [ \&tk_cm_titelwahl, $ax, $ay, $objekt, $element, $nr ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nbeschriftungsausrichtung ändern",
                               -command     => [ \&tk_cm_titelausrichtung, $ax, $ay, $objekt ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nbeschriftungsfarbe ändern (kleine Farbauswahl)",
                               -command     => [ \&Graph::GED::TkTools::farbwahl,
                                                 $Konfig{mw},
                                                 $ax, $ay,
                                                 sub {
                                                     Graph::GED::Logic::aenderung();
                                                     $objekt->titelfarbe($_[0]);
                                                     refresh();
                                                 },
                                               ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nbeschriftungsfarbe ändern (große Farbauswahl)",
                               -command     => sub {
                                                   my $c = $Konfig{mw}->chooseColor(-title
                                                       => "Bitte die Farbe für $element $nr"
                                                          . " auswählen:");
                                                   $objekt->titelfarbe($c) if $c;
                                                   refresh();
                                               },
                               -font        => $Konfig{menufont},
                              ],
                              '-',
                              ["command"    => "${element}nwert ändern",
                               -command     => [ \&tk_cm_wertwahl, $ax, $ay, $objekt, $element, $nr ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nwertausrichtung ändern",
                               -command     => [ \&tk_cm_wertausrichtung, $ax, $ay, $objekt ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nwertfarbe ändern (kleine Farbauswahl)",
                               -command     => [ \&Graph::GED::TkTools::farbwahl,
                                                 $Konfig{mw},
                                                 $ax, $ay,
                                                 sub {
                                                     Graph::GED::Logic::aenderung();
                                                     $objekt->gewichtfarbe($_[0]);
                                                     refresh();
                                                 },
                                               ],
                               -font        => $Konfig{menufont},
                              ],
                              ["command"    => "${element}nwertfarbe ändern (große Farbauswahl)",
                               -command     => sub {
                                                   my $c = $Konfig{mw}->chooseColor(-title
                                                       => "Bitte die Farbe für $element $nr"
                                                          . " auswählen:");
                                                   $objekt->gewichtfarbe($c) if $c;
                                                   refresh();
                                               },
                               -font        => $Konfig{menufont},
                              ],
                              '-',
                              ($ek == 1 && $Graph->gitterzeigen() ?
                              ["command"    => "${element} in Gitter einrasten",
                               -command     => sub { Graph::GED::Logic::aenderung();
                                                     my ($x, $y) = Graph::GED::Logic::raste_ein($objekt->x(), $objekt->y());
                                                     $objekt->x($x);
                                                     $objekt->y($y);
                                                     refresh();
                                                 },
                               -font        => $Konfig{menufont},
                              ] : () ),
                              ($ek == 1 ? # nur für Ecken:
                              ["command"    => "${element} mergen",
                               -command     => sub { Graph::GED::Logic::aenderung();
                                                     $Graph->ecke_mergen($nr);
                                                     refresh();
                                                 },
                               -font        => $Konfig{menufont},
                              ] : () ),
                              ["command"    => "${element} löschen",
                               -command     => sub { Graph::GED::Logic::aenderung();
                                                     $ek == 1 ? $Graph->ecke_loeschen($nr)
                                                              : $Graph->kante_loeschen($nr);
                                                     refresh();
                                                 },
                               -font        => $Konfig{menufont},
                              ],
                                        ],
                         );
    }
    $cmenu->post($ax, $ay);
    push @{ $Konfig{Contextmenu} }, $cmenu;
    $cmenu->bind(
        '<FocusOut>',
        sub {
            if (defined $Konfig{Contextmenu}) {
                $Konfig{Contextmenu}->destroy;
                $Konfig{Contextmenu} = undef;
            }
        }
    );
    #$cmenu->bind(
    #    '<Escape>',
    #    sub {
    #        if (defined $Konfig{Contextmenu}) {
    #            $Konfig{Contextmenu}->destroy;
    #            $Konfig{Contextmenu} = undef;
    #        }
    #    }
    #);
} # sub Graph::GED::Gui::context_menu


sub tk_cm_groessenwahl ($$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ g r o e s s e n w a h l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Größenauswahl für das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Größe gesetzt werden soll (Ecke)
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt) = @_;

    Graph::GED::Logic::aenderung();

    print "tk_cm_groessenwahl ($ax, $ay) Objekt $objekt\n"
        if $Konfig{debug};

    my @radien = (qw/  3   4   5   6
                       7   8   9  10
                      12  14  16  18
                      20  25  30  35
                      40  50  75 100/);
    my $index = 0;
    my $r = $objekt->radius() || $Graph->radius();
    for (0..$#radien) {
        if ($r == $radien[$_]) {
            $index = $_;
            last;
        }
    }
    my $radius = $radien[$index];

    my $menu = $Konfig{mw}->Menu(-tearoff => 0);

    for (@radien) {
        $menu->radiobutton(
            -label       => $_,
            -variable    => \$radius,
            -command     => sub {
                                  $objekt->radius($radius);
                                  refresh();
                                },
            -columnbreak => 0,
        );
    }

    for (0..$#radien-1) {
        $menu->entryconfigure($_, -columnbreak => 1) unless $_ % 5;
    }

    $menu->post($ax, $ay);
} # sub Graph::GED::Gui::tk_cm_groessenwahl


sub tk_cm_kantenformwahl ($$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ k a n t e n f o r m w a h l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Formauswahl für die Kante für das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Größe gesetzt werden soll (Kante)
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt) = @_;

    Graph::GED::Logic::aenderung();

    print "tk_cm_kantenformwahl ($ax, $ay) Objekt $objekt\n" if $Konfig{debug};

    my $f    = $Linientypen{ $objekt->form() }; # muss gesetzt sein!
    my $menu = $Konfig{mw}->Menu(-tearoff => 0);

    for (qw/k_ins k_inh k_inz k_inb/) {
        $menu->radiobutton(
            -label       => $Linientypen{$_},
            -variable    => \$f,
            -command     => sub {
                                  my $key;
                                  for (keys %Linientypen) {
                                      $key = $_ if $f eq $Linientypen{$_};
                                  }
                                  if ($key) {
                                      $objekt->form($key);
                                      refresh();
                                  }
                                  else {
                                      die "Interner Fehler bei Kantenformwahl";
                                  }
                                },
        );
    }

    $menu->post($ax, $ay);
} # sub Graph::GED::Gui::tk_cm_kantenformwahl


sub tk_cm_kantenstilwahl ($$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ k a n t e n s t i l w a h l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Stilauswahl für die Kante für das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Größe gesetzt werden soll (Kante)
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt) = @_;

    Graph::GED::Logic::aenderung();

    print "tk_cm_kantenstilwahl ($ax, $ay) Objekt $objekt\n"
        if $Konfig{debug};

    my @stile = (qw/normal gestrichelt gepunktet/);
    my $s     = $objekt->stil();
    my $menu  = $Konfig{mw}->Menu(-tearoff => 0);

    for (@stile) {
        $menu->radiobutton(
            -label       => $_,
            -variable    => \$s,
            -command     => sub {
                                  $objekt->stil($s);
                                  refresh();
                                },
        );
    }

    $menu->post($ax, $ay);
} # sub Graph::GED::Gui::tk_cm_kantenstilwahl


sub tk_cm_kantendicke ($$$) {
    #--------------------------------------------------------------------------
    # sub          : tk_cm_kantendicke
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Wahl der Kantendicke für das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Größe gesetzt werden soll (Ecke)
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    # 0.0.2 - 16.01.2005 - CD - Auch für Default-Kantendicke verwendbar (das
    #                           Objekt ist dann vom Typ $Graph2D).
    # 0.0.3 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt) = @_;

    Graph::GED::Logic::aenderung();

    print "tk_cm_kantendicke ($ax, $ay) Objekt $objekt\n" if $Konfig{debug};

    my @dicken = (qw/  1   2   3   4   5
                       6   7   8   9  10
                      11  12  14  16  17
                      18  19  20  25  30
                      35  40  50  75 100/);
    my $d      = $objekt->isa('Graph::GED::Graph2D') ? $objekt->kantendicke()
                                         : $objekt->dicke();
    my $menu   = $Konfig{mw}->Menu(-tearoff => 0);

    for (@dicken) {
        $menu->radiobutton(
            -label       => $_,
            -variable    => \$d,
            -command     => sub {
                                  if ($objekt->isa('Graph::GED::Graph2D')) {
                                      $objekt->kantendicke($d);
                                  }
                                  else {
                                      $objekt->dicke($d);
                                  }
                                  refresh();
                                },
            -columnbreak => 0,
        );
    }

    for (0..$#dicken) {
        $menu->entryconfigure($_, -columnbreak => 1) unless $_ % 5;
    }

    $menu->post($ax, $ay);

} # sub Graph::GED::Gui::tk_cm_kantendicke


sub tk_cm_titelwahl ($$$$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ t i t e l w a h l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Titelauswahl für das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Größe gesetzt werden soll (Ecke)
    #                4) Ecke oder Kante
    #                5) Nummer des Objekts
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt, $element, $nr) = @_;

    Graph::GED::Logic::aenderung();

    print "tk_cm_titelwahl ($ax, $ay) Objekt $objekt\n" if $Konfig{debug};

    my $t = $Konfig{mw}->Toplevel();
    $t->transient($Konfig{mw});
    $t->overrideredirect(1);
    $t->geometry("+$ax+$ay");
    #
    #$t->title("Bitte Beschreibung für $element $nr eingeben");
    # braucht man nicht wegen overrideredirect(1)
    $t->label("Bitte Beschreibung für $element $nr eingeben");

    my $tvar = $objekt->titel();
    my $txt = $t->Entry(-width        => 20,
                        -textvariable => \$tvar,
                       )
                 ->pack(-side         => 'left',
                        -fill         => 'x',
                        -expand       => 0,
                       );
    my $ok = $t->Button(-text        => 'OK',
                        -command     => sub {
                                             $objekt->titel($tvar);
                                             $t->destroy();
                                             $Konfig{mw}->focus();
                                             refresh();
                                            },
                        -default     => 'active',
                        -padx        => 15,
                       )
                 ->pack(-expand      => '0',
                        -fill        => 'none',
                        -side        => 'left',
                       );

    $t->bind('<Return>', sub{$ok->invoke()});

    $t->grab();
    $t->focus();
    $txt->focus();

} # sub Graph::GED::Gui::tk_cm_titelwahl


sub tk_cm_titelausrichtung ($$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ t i t e l a u s r i c h t u n g
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Titelausrichtung wählen für das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Größe gesetzt werden soll (Ecke)
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt) = @_;

    Graph::GED::Logic::aenderung();

    print "tk_cm_titelausrichtung ($ax, $ay) Objekt $objekt\n"
        if $Konfig{debug};

    my @positionen = (qw/NW W  SW
                         N  *  S
                         NO O  SO/);
    my $index = 0;
    my $p = $objekt->titelpos() || $positionen[0];
    for (0..$#positionen) {
        if ($p eq $positionen[$_]) {
            $index = $_;
            last;
        }
    }
    my $pos = $positionen[$index];

    my $menu = $Konfig{mw}->Menu(-tearoff => 0);

    for (@positionen) {
        $menu->radiobutton(
            -label       => $_,
            -variable    => \$pos,
            -command     => sub {
                                  $objekt->titelpos($pos) unless $pos eq '*';
                                  refresh();
                                },
            -columnbreak => 0,
        );
    }

    for (0..$#positionen) {
        $menu->entryconfigure($_, -columnbreak => 1) unless $_ % 3;
    }

    $menu->post($ax, $ay);

} # sub Graph::GED::Gui::tk_cm_titelausrichtung


sub tk_cm_wertwahl ($$$$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ w e r t w a h l
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Titelauswahl für das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Größe gesetzt werden soll (Ecke)
    #                4) Ecke oder Kante
    #                5) Nummer des Objekts
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt, $element, $nr) = @_;

    Graph::GED::Logic::aenderung();

    print "tk_cm_wertwahl_klein ($ax, $ay) Objekt $objekt\n" if $Konfig{debug};

    my $t = $Konfig{mw}->Toplevel();
    $t->transient($Konfig{mw});
    $t->overrideredirect(1);
    $t->geometry("+$ax+$ay");
    #
    #$t->title("Bitte Wert für $element $nr eingeben");
    # braucht man nicht wegen overrideredirect(1)
    $t->label("Bitte Wert für $element $nr eingeben");

    my $tvar = $objekt->gewicht();
    my $txt = $t->Entry(-width        => 20,
                        -textvariable => \$tvar,
                       )
                 ->pack(-side         => 'left',
                        -fill         => 'x',
                        -expand       => 0,
                       );
    my $ok = $t->Button(-text        => 'OK',
                        -command     => sub {
                                             $objekt->gewicht($tvar);
                                             #use Data::Dumper; print Dumper $Graph;
                                             $t->destroy();
                                             $Konfig{mw}->focus();
                                             refresh();
                                            },
                        -default     => 'active',
                        -padx        => 15,
                       )
                 ->pack(-expand      => '0',
                        -fill        => 'none',
                        -side        => 'left',
                       );

    $t->bind('<Return>', sub{$ok->invoke()});

    $t->grab();
    $t->focus();
    $txt->focus();

} # sub Graph::GED::Gui::tk_cm_wertwahl


sub tk_cm_wertausrichtung ($$$) {
    #--------------------------------------------------------------------------
    # sub          : t k _ c m _ w e r t a u s r i c h t u n g
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Wertausrichtung wählen für das Kontextmenue.
    # Parameter    : 1) absolute x-Koordinate
    #                2) absolute y-Koordinate
    #                3) Objekt in dem Größe gesetzt werden soll (Ecke)
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 30.12.2004 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    my ($ax, $ay, $objekt) = @_;

    Graph::GED::Logic::aenderung();

    print "tk_cm_wertausrichtung ($ax, $ay) Objekt $objekt\n"
        if $Konfig{debug};

    my @positionen = (qw/NW W  SW
                         N  *  S
                         NO O  SO/);
    my $index = 0;
    my $p = $objekt->gewichtpos() || $positionen[8];
    for (0..$#positionen) {
        if ($p eq $positionen[$_]) {
            $index = $_;
            last;
        }
    }
    my $pos = $positionen[$index];

    my $menu = $Konfig{mw}->Menu(-tearoff => 0);

    for (@positionen) {
        $menu->radiobutton(
            -label       => $_,
            -variable    => \$pos,
            -command     => sub {
                                  $objekt->gewichtpos($pos) unless $pos eq '*';
                                  refresh();
                                },
            -columnbreak => 0,
        );
    }

    for (0..$#positionen) {
        $menu->entryconfigure($_, -columnbreak => 1) unless $_ % 3;
    }

    $menu->post($ax, $ay);

} # sub Graph::GED::Gui::tk_cm_wertausrichtung


sub clear_context_menu () {
    #--------------------------------------------------------------------------
    # sub          : c l e a r _ c o n t e x t _ m e n u
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Entfernt alle Kontextmenüs (nötig wegen Linux).
    # Parameter    : Dateiname
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 17.01.2005 - CD - Erstellt
    # 0.0.2 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    if (exists $Konfig{Contextmenu}) {
        for my $i (reverse 0..$#{$Konfig{Contextmenu}}) {
            my ($c) = splice @{ $Konfig{Contextmenu} }, $i, 1;
            $c->destroy if defined $c;
            $c = undef;
        }
    }
} # sub Graph::GED::Gui::clear_context_menu


sub animation_schalten () {
    #--------------------------------------------------------------------------
    # sub          : a n i m a t i o n _ s c h a l t e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Animation auf angegebenen Wert schalten.
    # Parameter    : Nummer (ev. mit plus oder minus davor)
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 19.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    my ($w, $n) = @_;
    $n = $w unless defined $n; # w : widgetinfo per bind

    print "animation_schalten, n = $n\n" if $Debug;

    #--------------------------------------------------------------------------
    # Plus / minus ausrechnen:
    #--------------------------------------------------------------------------
    if ($n =~ /^[-+]/) {
        my $actual = $Konfig{animationskala}->get();
        my ($operation, $zahl) = $n =~ m~^([-+])(\d+)~;
        return unless defined $operation and defined $zahl;
        if ($operation eq '+') {
            $n = $actual + $zahl;
        }
        elsif ($operation eq '-') {
            $n = $actual - $zahl;
        }
        else {
            return;
        }
        $Konfig{animationskala}->set($n);
        return; # dadurch wird diese Funktion nochmal aufgerufen mit der
                #gesetzten Zahl und durchläuft den unteren Teil.
    }

    #print "n = $n\n";

    #Graph::GED::Logic::aenderung();

    #print "n = $n, graphen: ", $Animation->number_of_graphs(), ", historien: ", $Animation->number_of_histories(), "\n";

    return if $n < 1                                 or
              $n > $Animation->number_of_graphs()    or
              $n > $Animation->number_of_histories()  ;

    $Konfig{animationskala}->set($n); # ist nötig, falls nicht vorher mit +1
                                      # oder -1 geschaltet wurde!
    $Graph   = $Animation->graph  ($n-1);
#use Data::Dumper;print Data::Dumper::Dumper $Graph;
#print "zeige Graph Nr $n : $Graph\n";
    $History = $Animation->history($n-1);
    $Graph->r2d();

    set_title();

    refresh();
} # sub Graph::GED::Gui::animation_schalten


sub animations_verwaltung () {
    #--------------------------------------------------------------------------
    # sub          : a n i m a t i o n _ v e r w a l t u n g
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Animation auf angegebenen Wert schalten.
    # Parameter    : Nummer (ev. mit plus oder minus davor)
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 19.01.2005 - CD - Erstellt
    # 0.0.2 - 21.01.2005 - CD - Anfänge der Darstellung
    #--------------------------------------------------------------------------

    $Konfig{mw}->Busy(-recurse => 1);
    my $actual = $Konfig{animationskala}->get(); 

    # Großes Toplevel mit 3 Spalten, in denen die Graphen dargestellt werden.
    # Dafür Original-Kanvas sichern, danach immer die einzel Canvas als
    # normales Canvas einsetzen, den Graphen sichern, dann skalieren und
    # zentrieren, dann refreshen (=einzeichnen), hinterher Graph wieder
    # herstellen.
    # Am Ende Canvas wieder herstellen.

    #--------------------------------------------------------------------------
    # Bindungen an die Canvas lösen:
    #--------------------------------------------------------------------------
    $Konfig{canvas}->Tk::bind("<Motion>",    "");
    $Konfig{canvas}->Tk::bind("<Button-1>",  "");

    #--------------------------------------------------------------------------
    # Neues Toplevel-Fenster erzeugen, das den Text anzeigt:
    #--------------------------------------------------------------------------
    my $tl = $Konfig{mw}->Toplevel();
    $tl->title('Animationsverwaltung');
    $tl->Label(
        -text   => 'Animationsübersicht:',
       #-anchor => 'w',
    )->pack(
        -anchor => 'w',
    );

    #--------------------------------------------------------------------------
    # Rahmen, in dem die Animationsbilder gezeigt werden:
    #--------------------------------------------------------------------------
    my $x = int(($Graph::GED::Graph2D::MaxX - $Graph::GED::Graph2D::MinX)/4);
    my $y = int(($Graph::GED::Graph2D::MaxY - $Graph::GED::Graph2D::MinY)/4);
    my $z = $Animation->number_of_graphs() < 7
                ? $Animation->number_of_graphs()
                : 7;
            # Anzahl Graphen in einer Reihe
    my $v = int($Animation->number_of_graphs() / $z) * ($y + 10 + 5);
    my $aniframe = $tl->Scrolled(
        'Frame',
        -scrollbars => 'osoe',
        -width      => $z * ($x + 10 + 5),
        -height     => $v > 400 ? 400 : $v,
    )->pack(
        -expand => '1',
        -fill   => 'both',
        -side   => 'top',
    );

    #--------------------------------------------------------------------------
    # Animationsbilder darstellen:
    #--------------------------------------------------------------------------
    {
        my $savegraph  = $Graph;                      # Aktuellen Graphen
                                                      # sichern
        my $savecanvas = $Konfig{canvas};             # Aktuelle Canvas
                                                      # sichern
        my @canvasses;                                # Die Canvasses für die
                                                      # Graphendarstellungen
        my @frames;                                   # Die Rahmen um die
                                                      # Canvasse.
        my @graphs     = @{ $Animation->_graphs() };  # Darzustellende Graphen

        for (my $i=0; $i<=$#graphs; $i+=$z) {
            #------------------------------------------------------------------
            # Zeilenframe:
            #------------------------------------------------------------------
            my $frm = $aniframe->Frame(
            )->pack(
                -expand => '1',
                -fill   => 'x',
                -side   => 'top',
            );

            #------------------------------------------------------------------
            # Canvasses und Rahmen erstellen und Graphen einzeichnen:
            #------------------------------------------------------------------
            for my $j ($i .. $i+$z-1) {
                last if $j > $#graphs;
                push @frames, $frm->Frame(
                    -borderwidth => 5,
                    -background  => $j == $actual-1 ? 'black' : 'white',
                )->pack(
                    -expand => '0',
                    -fill   => 'both',
                    -side   => 'left',
                );
                push @canvasses, $frames[$j]->Canvas(
                    -width  => $x,
                    -height => $y,
                )->pack(
                );
                $canvasses[$j]->Tk::bind(
                    '<Button-1>',
                    [ sub {
                        $_->configure(-background => 'white') for @frames;
                        $frames[$_[1]]->configure(-background => 'black');
                      }, $j
                    ],
                );
                $Graph          = $graphs[$j]->clone();
                $Konfig{canvas} = $canvasses[$j] ;
                
                ($Graph::GED::Graph2D::MinX, $Graph::GED::Graph2D::MinY,
                 $Graph::GED::Graph2D::MaxX, $Graph::GED::Graph2D::MaxY)
                = (0, 0, $x, $y);
                $Graph->zentrieren_skalieren();
                refresh();
            }
        }

        # Graphen und Canvas wieder herstellen:
        $Graph          = $savegraph;
        $Konfig{canvas} = $savecanvas ;
        refresh();
    }

    #--------------------------------------------------------------------------
    # Buttons am unteren Rand darstellen:
    #--------------------------------------------------------------------------
    my $f2 = $tl->Frame(
    )->pack(
        -expand => '0',
        -fill   => 'x',
        -side   => 'top',
    );
    my $ok = $f2->Button(
        -text        => 'OK',
        -command     =>
            sub {
                $tl->destroy();
                #$Konfig{mw}
                #  ->bind('<Escape>', \&beenden);
                $Konfig{mw}->focus();
            },
        -default     => 'active',
        -padx        => 15,
    )->pack(
        -expand      => '0',
        -fill        => 'none',
        -side        => 'right',
    );
    $f2->Button(
        -text        => 'Kopieren',
    )->pack(
        -expand      => '0',
        -fill        => 'none',
        -side        => 'right',
    );
    $f2->Button(
        -text        => 'Entfernen',
    )->pack(
        -expand      => '0',
        -fill        => 'none',
        -side        => 'right',
    );
    $f2->Button(
        -text        => 'Verschieben',
    )->pack(
        -expand      => '0',
        -fill        => 'none',
        -side        => 'right',
    );
    $f2->Button(
        -text        => 'Austausch',
    )->pack(
        -expand      => '0',
        -fill        => 'none',
        -side        => 'right',
    );

    #--------------------------------------------------------------------------
    # Bindungen zum $MW für Escape lösen und an Löschen dieses Fensters binden.
    # (Beim Beenden des Fensters wird ESC wieder an $MW gebunden (siehe oben).)
    #--------------------------------------------------------------------------
    #$Konfig{mw}->bind ('<Escape>', '');

    $tl->bind('<Return>',     sub{ $ok->invoke() }              );
    $tl->bind('<Escape>',     sub{ $ok->invoke() }              );
    #$tl->bind('<Home>',       sub { $aniframe->yviewMoveto(0) } );
    #$tl->bind('<End>',        sub { $aniframe->yviewMoveto(1) } );

    #$tl->bind('<Left>',       sub { $aniframe->yviewScroll(-1, 'units') } );
    #$tl->bind('<Up>',         sub { $aniframe->yviewScroll(-1, 'units') } );
    #$tl->bind('<Down>',       sub { $aniframe->yviewScroll( 1, 'units') } );
    #$tl->bind('<Right>',      sub { $aniframe->yviewScroll( 1, 'units') } );
    #$tl->bind('<Alt-Up>',     sub { $aniframe->yviewScroll(-3, 'units') } );
    #$tl->bind('<Alt-Down>',   sub { $aniframe->yviewScroll( 3, 'units') } );
    #$tl->bind('<Prior>',      sub { $aniframe->yviewScroll(-1, 'pages') } );
    #$tl->bind('<Next>',       sub { $aniframe->yviewScroll( 1, 'pages') } );
    #$tl->bind('<MouseWheel>', [ sub { $aniframe->yviewScroll(-($_[1]/120)*3, 'units') }, Tk::Ev('D') ] );


    #--------------------------------------------------------------------------
    # Rest:
    # (grab verhindert das zurückschalten auf $WM per Klick, focus sorgt dafür,
    # dass $tl auch wirklich aktiv ist.)
    #--------------------------------------------------------------------------
    $Konfig{mw}->Unbusy();
    $tl->grab();
    $tl->focus();
    
    #--------------------------------------------------------------------------
    # Bindungen an die Canvas wieder herstellen:
    #--------------------------------------------------------------------------
    $Konfig{mw}->after(1000, \&bind_start);

} # sub Graph::GED::Gui::animations_verwaltung


sub set_title () {
    #--------------------------------------------------------------------------
    # sub          : s e t _ t i t l e
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Programmtitel setzen
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 19.01.2005 - CD - Erstellt
    #--------------------------------------------------------------------------

    my $gn = $Graph->dateiname();
    my $an = $Animation->filename();

    $gn = '   -   (' . $gn . ')' if length $gn;
    $an = '   -   [' . $an . ']' if length $an;
    
    $Konfig{mw}->title('G - E - D   ' . $Konfig{version} . $gn . $an);

} # sub Graph::GED::Gui::set_title


sub tk_beenden () {
    #--------------------------------------------------------------------------
    # sub          : t k _ b e e n d e n
    #--------------------------------------------------------------------------
    # Autor        : Christian Dühl
    # Aufgabe      : Programm beenden
    # Parameter    : keine
    # Rückgabewert : keiner
    #--------------------------------------------------------------------------
    # 0.0.1 - 06.02.2004 - CD - Erstellt
    # 0.0.2 - 03.01.2005 - CD - Bei Änderungen nachfragen.
    # 0.0.3 - 18.01.2005 - CD - Verschoben nach Graph::GED::Gui.
    #--------------------------------------------------------------------------

    if ($Konfig{changed} eq '*') {
        my $icon = $Konfig{mw}->Photo(
            -file   => "$FindBin::RealBin/res/ged.bmp",
            -height => 32,
            -width  => 32,
        );
        my $dialog = $Konfig{mw}->Dialog(
            -text           => 'Sie haben nicht gesicherte Änderungen! Was wollen sie tun?',
            #-bitmap         => 'info',
            -image          => $icon,
            -title          => 'GED beenden?',
            -default_button => 'GED Beenden',
            -buttons        => [ 'GED Beenden', 'Speichern', 'Abbrechen' ],
            #-font           => '{Courier New} 10 {normal}',
            -font           => '{Times New Roman} 12 {normal}',
        );
        my $antwort = $dialog->Show(-global);
        return if $antwort eq 'Abbrechen';
        speichere_datei() if $antwort eq 'Speichern';
        $Konfig{mw}->destroy();
    }
    else {
        $Konfig{mw}->destroy();
    }
} # sub Graph::GED::Gui::tk_beenden

1;


__END__

=head1 NAME

Graph::GED::Gui.pm - a module with the Tk-Functions of GED

=head1 FUNCTIONS

=over

=item C<start>

    Starts the GUI.

=item C<tk_start>

    Builds the main window definition.

=item C<add_file_menu>

    Adds the file menu (for file history reasons in an extra function).

=item C<bind_start>

    Starts the canvas bindings.

=item C<zeichnen_anfang>

    Starts the drawing (edges, vertices)

=item C<aendere_item>

    Handles things to do while drawing.

=item C<zeichnen_ende>

    Stops the drawing.

=item C<refresh>

    Refreshes the canvas.

=item C<clear>

    Clears the graph.

=item C<zeichne_ecke>

    Draws a vertex.

=item C<zeichne_kante>

    Draws an edge.

=item C<lade_datei>

    Loads a file.

=item C<speichere_datei>

    Save file

=item C<speichere_datei_als>

    Save file as

=item C<lade_animation>

    Loads animation.

=item C<speichere_animation>

    Save animation

=item C<speichere_animation_als>

    Save animation as

=item C<lade_hintergrundbild>

    loads backgroud picture

=item C<export_nach_latex>

    Starts the export to LaTeX file.

=item C<export_nach_postscript>

    Starts the export to a postscript file.

=item C<dump_eingeben>

    Allows to enter ged-Dump.

=item C<zeige_daten_an>

    shows data

=item C<zeige_daten_an_dumper>

    shows data with dumper

=item C<hilfe>

    shows the help

=item C<graph_erkl>

    Shows a explanation to graphs.

=item C<history>

    Shows the program history.

=item C<todo>

    Shows the todo list.

=item C<show_perldoc>

    Shows the perldoc of the given file.

=item C<ueber_ged>

    Shows dialog box "about GED".

=item C<canvas_groesse>

    Determines the size of the canvas.

=item C<waehle_eckenfarbe>

    Choose vertex color.

=item C<waehle_kantenfarbe>

    Choose edge color.
    
=item C<historie_schalter_aktualisieren>

    Acualize history buttons.

=item C<context_menu>

    Show context menu.

=item C<tk_cm_groessenwahl>

    Choose size.

=item C<tk_cm_kantenformwahl>

    Choose edge form.

=item C<tk_cm_kantenstilwahl>

    Choose edge style.

=item C<tk_cm_kantendicke>

    Choose width of edges.

=item C<tk_cm_titelwahl>

    Choose title.

=item C<tk_cm_titelausrichtung>

    Choose title position.

=item C<tk_cm_wertwahl>

    Choose weight.

=item C<tk_cm_wertausrichtung>

    Choose weight position.

=item C<clear_context_menu>

    Clears all contexrt menues.

=item C<animation_schalten>

    Switches the animation to the given number.

=item C<set_title>

    Sets the title of GED's window.

=item C<tk_beenden>

    Quit Tk.

=back

=head1 COPYRIGHT

Copyright 2005 Christian Duehl. All rights reserved.

This library is free software. You can redistribute it and/or
modify it under the same terms as perl itself.

=cut
