package de.duehl.math.geometry;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

/**
 * Diese Klasse steht für einen Kreis.
 *
 * @version 1.01     2013-05-21
 * @author Christian Dühl
 */

public class Circle {

    /** Mittelpunkt des Kreises */
    private Point center;

    /** Radius des Kreises */
    private double radius;

    /** Konstruktor */
    public Circle() {
        this(new Point(), 0);
    }

    /**
     * Konstruktor
     *
     * @param center
     *            Mittelpunkt des Kreises
     * @param radius
     *            Radius des Kreises
     */
    public Circle(Point center, double radius) {
        this.center = center;
        this.radius = radius;
    }

    /** Getter für den Mittelpunkt. */
    public Point getCenter() {
        return center;
    }

    /** Setter für den Mittelpunkt. */
    public void setCenter(Point center) {
        this.center = center;
    }

    /** Getter für den Radius. */
    public double getRadius() {
        return radius;
    }

    /** Setter für den Radius. */
    public void setRadius(double radius) {
        this.radius = radius;
    }

    /**
     * Ermittelt, ob der übergebene Punkt auf der Kreislinie oder innerhalb
     * davon liegt.
     *
     * @param point
     *            Zu überprüfender Punkt.
     * @return Wahrheitswert.
     */
    public boolean incloses(Point point) {
        double pointX = point.getX();
        double pointY = point.getY();

        double centerX = center.getX();
        double centerY = center.getY();

        /*
         * Falls der Klick außerhalb des umschließenden Quadrats ist, gleich
         * abbrechen:
         */
        if (pointX < centerX-radius || pointX > centerX+radius
                || pointY < centerY-radius || pointY > centerY+radius) {
            return false;
        }

        /* Nach Pythagoras: */
        double dx = pointX - centerX; // Vorzeichen egal, da quadriert wird
        double dy = pointY - centerY; // Vorzeichen egal, da quadriert wird
        if (dx*dx + dy*dy <= radius*radius) {
            return true;
        }
        else {
            return false;
        }
    }


    /* --- generierte Methoden: --- */


    @Override
    public String toString() {
        return "Circle [center=" + center + ", radius=" + radius + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((center == null) ? 0 : center.hashCode());
        long temp;
        temp = Double.doubleToLongBits(radius);
        result = prime * result + (int) (temp ^ (temp >>> 32));
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        Circle other = (Circle) obj;
        if (center == null) {
            if (other.center != null) {
                return false;
            }
        }
        else if (!center.equals(other.center)) {
            return false;
        }
        if (Double.doubleToLongBits(radius) != Double
                .doubleToLongBits(other.radius)) {
            return false;
        }
        return true;
    }

    /** Getter für den Radius als Ganzzahlwert. */
    public int getRadiusAsInteger() {
        return (int) Math.round(radius);
    }

}
