package de.duehl.math.numbertheory;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

/**
 * Diese Klasse stellt ein Intervall von ganzen Zahlen dar. Beide Intervallgrenzen sind
 * einschließlich zu verstehen.
 *
 * @version 1.01     2016-08-17
 * @author Christian Dühl
 */

public class Interval {

    /** Einschließliche untere Intervallgrenze. */
    private final int from;

    /** Einschließliche obere Intervallgrenze. */
    private final int to;

    /**
     * Konstruktor.
     *
     * @param from
     *            Einschließliche untere Intervallgrenze.
     * @param to
     *            Einschließliche obere Intervallgrenze.
     */
    public Interval(int from, int to) {
        if (from > to) {
            throw new RuntimeException("'from' muss kleiner gleich 'to' sein");
        }
        this.from = from;
        this.to = to;
    }

    /** Getter für die einschließliche untere Intervallgrenze. */
    public int getFrom() {
        return from;
    }

    /** Getter für die einschließliche obere Intervallgrenze. */
    public int getTo() {
        return to;
    }

    /** Gibt an, ob sich dieses Intervall mit dem übergebenen überschneidet. */
    public boolean intersect(Interval that) {
        if (this.from >= that.from && this.from <= that.to) {
            return true;
        }
        if (that.from >= this.from && that.from <= this.to) {
            return true;
        }
        return false;
    }

    /**
     * Gibt an, ob sich dieses Intervall mit dem übergebenen überschneidet, aber vor dem anderen
     * Intervall liegt.
     */
    public boolean intersectBefore(Interval that) {
        return this.from < that.from && that.from <= this.to;
    }

    /** Gibt an, ob dieses Intervall das übergebene umfasst / enthält. */
    public boolean including(Interval that) {
        return this.from <= that.from && this.to >= that.to;
    }

    /** Erzeugt eine Darstellung als String. */
    @Override
    public String toString() {
        return "[" + from + ", " + to + "]";
    }

}
