package de.duehl.math.numbertheory;

/*
 * Copyright 2016 Christian Dühl. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or
 * modify it under the same terms as perl:
 *
 * general:  http://dev.perl.org/licenses/
 * GPL:      http://dev.perl.org/licenses/gpl1.html
 * artistic: http://dev.perl.org/licenses/artistic.html
 */

import static org.junit.Assert.*;

import org.junit.Test;

import de.duehl.math.numbertheory.Interval;

public class IntervalTest {

    @Test
    public void create() {
        Interval interval = new Interval(0, 100);
        assertNotNull(interval);
    }

    @Test (expected = RuntimeException.class)
    public void failWithWrongInitialisationValues() {
        new Interval(100, 0);
    }

    @Test
    public void getFrom() {
        Interval interval = new Interval(0, 100);
        int actual = interval.getFrom();
        int expected = 0;
        assertEquals(expected, actual);
    }

    @Test
    public void getTo() {
        Interval interval = new Interval(0, 100);
        int actual = interval.getTo();
        int expected = 100;
        assertEquals(expected, actual);
    }

    @Test
    public void toStringTest() {
        Interval interval = new Interval(0, 100);
        String actual = interval.toString();
        String expected = "[0, 100]";
        assertEquals(expected, actual);
    }

    @Test
    public void intersect1() {
        Interval interval1 = new Interval(0, 100);
        Interval interval2 = new Interval(50, 150);
        assertTrue(interval1.intersect(interval2));
        assertTrue(interval2.intersect(interval1));
    }

    @Test
    public void intersect2() {
        Interval interval1 = new Interval(0, 100);
        Interval interval2 = new Interval(100, 150);
        assertTrue(interval1.intersect(interval2));
        assertTrue(interval2.intersect(interval1));
    }

    @Test
    public void intersect3() {
        Interval interval1 = new Interval(0, 100);
        Interval interval2 = new Interval(50, 50);
        assertTrue(interval1.intersect(interval2));
        assertTrue(interval2.intersect(interval1));
    }

    @Test
    public void intersect4() {
        Interval interval1 = new Interval(50, 50);
        Interval interval2 = new Interval(50, 50);
        assertTrue(interval1.intersect(interval2));
        assertTrue(interval2.intersect(interval1));
    }

    @Test
    public void noIintersect1() {
        Interval interval1 = new Interval(0, 100);
        Interval interval2 = new Interval(101, 150);
        assertFalse(interval1.intersect(interval2));
        assertFalse(interval2.intersect(interval1));
    }

    @Test
    public void noIintersect2() {
        Interval interval1 = new Interval(0, 0);
        Interval interval2 = new Interval(1, 1);
        assertFalse(interval1.intersect(interval2));
        assertFalse(interval2.intersect(interval1));
    }

    @Test
    public void including1() {
        Interval interval1 = new Interval(0, 100);
        Interval interval2 = new Interval(50, 75);
        assertTrue(interval1.including(interval2));
    }

    @Test
    public void including2() {
        Interval interval1 = new Interval(0, 100);
        Interval interval2 = new Interval(0, 10);
        assertTrue(interval1.including(interval2));
    }

    @Test
    public void including3() {
        Interval interval1 = new Interval(0, 100);
        Interval interval2 = new Interval(90, 100);
        assertTrue(interval1.including(interval2));
    }

    @Test
    public void including4() {
        Interval interval1 = new Interval(0, 100);
        Interval interval2 = new Interval(0, 100);
        assertTrue(interval1.including(interval2));
    }

    @Test
    public void including5() {
        Interval interval1 = new Interval(0, 0);
        Interval interval2 = new Interval(0, 0);
        assertTrue(interval1.including(interval2));
    }

    @Test
    public void noIncluding1() {
        Interval interval1 = new Interval(0, 100);
        Interval interval2 = new Interval(50, 150);
        assertFalse(interval1.including(interval2));
    }

    @Test
    public void intersectBefore1() {
        Interval interval1 = new Interval(0, 100);
        Interval interval2 = new Interval(50, 150);
        assertTrue(interval1.intersectBefore(interval2));
        assertFalse(interval2.intersectBefore(interval1));
    }

    @Test
    public void NointersectBefore1() {
        Interval interval1 = new Interval(0, 100);
        Interval interval2 = new Interval(0, 100);
        assertFalse(interval1.intersectBefore(interval2));
        assertFalse(interval2.intersectBefore(interval1));
    }

    @Test
    public void NointersectBeforePoints() {
        Interval interval1 = new Interval(0, 0);
        Interval interval2 = new Interval(0, 0);
        assertFalse(interval1.intersectBefore(interval2));
        assertFalse(interval2.intersectBefore(interval1));
    }

}
