package de.duehl.math.numbertheory;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

public class SixtytwoNumberingSystemTest {

    @Test
    public void getNumberOfDigits() {
        int actual = SixtytwoNumberingSystem.getNumberOfDigits();
        int expected = 62;
        assertEquals(expected, actual);
    }

    @Test
    public void getDigitForDecimalValueForDecimal0() {
        String actual = SixtytwoNumberingSystem.getDigitForDecimalValue(0);
        String expected = "0";
        assertEquals(expected, actual);
    }

    @Test
    public void getDigitForDecimalValueForDecimal12() {
        String actual = SixtytwoNumberingSystem.getDigitForDecimalValue(12);
        String expected = "c";
        assertEquals(expected, actual);
    }

    @Test
    public void getDigitForDecimalValueForDecimal35() {
        String actual = SixtytwoNumberingSystem.getDigitForDecimalValue(35);
        String expected = "z";
        assertEquals(expected, actual);
    }

    @Test
    public void getDigitForDecimalValueForDecimal36() {
        String actual = SixtytwoNumberingSystem.getDigitForDecimalValue(36);
        String expected = "A";
        assertEquals(expected, actual);
    }

    @Test
    public void getDigitForDecimalValueForDecimal61() {
        String actual = SixtytwoNumberingSystem.getDigitForDecimalValue(61);
        String expected = "Z";
        assertEquals(expected, actual);
    }

    @Test (expected = IllegalArgumentException.class)
    public void getDigitForDecimalValueForNegativeDecimal() {
        SixtytwoNumberingSystem.getDigitForDecimalValue(-1);
    }

    @Test (expected = IllegalArgumentException.class)
    public void getDigitForDecimalValueForTooBigDecimal() {
        SixtytwoNumberingSystem.getDigitForDecimalValue(SixtytwoNumberingSystem.getNumberOfDigits());
    }

    @Test
    public void getDecimalValueForDigitFor0() {
        String digit = "0";
        long actual = SixtytwoNumberingSystem.getDecimalValueForDigit(digit);
        long expected = 0;
        assertEquals(expected, actual);
    }

    @Test
    public void getDecimalValueForDigitFor9() {
        String digit = "9";
        long actual = SixtytwoNumberingSystem.getDecimalValueForDigit(digit);
        long expected = 9;
        assertEquals(expected, actual);
    }

    @Test
    public void getDecimalValueForDigitFora() {
        String digit = "a";
        long actual = SixtytwoNumberingSystem.getDecimalValueForDigit(digit);
        long expected = 10;
        assertEquals(expected, actual);
    }

    @Test
    public void getDecimalValueForDigitForz() {
        String digit = "z";
        long actual = SixtytwoNumberingSystem.getDecimalValueForDigit(digit);
        long expected = 35;
        assertEquals(expected, actual);
    }

    @Test
    public void getDecimalValueForDigitForA() {
        String digit = "A";
        long actual = SixtytwoNumberingSystem.getDecimalValueForDigit(digit);
        long expected = 36;
        assertEquals(expected, actual);
    }

    @Test
    public void getDecimalValueForDigitForZ() {
        String digit = "Z";
        long actual = SixtytwoNumberingSystem.getDecimalValueForDigit(digit);
        long expected = 61;
        assertEquals(expected, actual);
    }

    @Test
    public void checkCharsHaveNoDoublets() {
        List<String> seenDigits = new ArrayList<>();
        for (int decimal = 0; decimal < SixtytwoNumberingSystem.getNumberOfDigits(); ++decimal) {
            String digit = SixtytwoNumberingSystem.getDigitForDecimalValue(decimal);
            if (seenDigits.contains(digit)) {
                fail("Doppelte Ziffer " + digit + " in Zifferstring!");
            }
            seenDigits.add(digit);
        }
    }

    @Test
    public void decimalToSixtytwoDecimal0() {
        long decimalNumber = 0;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "0";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal1() {
        long decimalNumber = 1;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "1";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal9() {
        long decimalNumber = 9;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "9";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal10() {
        long decimalNumber = 10;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "a";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal35() {
        long decimalNumber = 35;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "z";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal36() {
        long decimalNumber = 36;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "A";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal61() {
        long decimalNumber = 61;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "Z";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal62() {
        long decimalNumber = 62;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "10";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal71() {
        long decimalNumber = 71;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "19";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal72() {
        long decimalNumber = 72;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "1a";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal123() {
        long decimalNumber = 123;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "1Z";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal124() {
        long decimalNumber = 124;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "20";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal619() {
        long decimalNumber = 619;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "9Z";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal620() {
        long decimalNumber = 620;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "a0";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal682() {
        long decimalNumber = 682;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "b0";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal744() {
        long decimalNumber = 744;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "c0";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal2170() {
        long decimalNumber = 2170;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "z0";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal2232() {
        long decimalNumber = 2232;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "A0";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal3782() {
        long decimalNumber = 3782;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "Z0";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal3844() {
        long decimalNumber = 3844;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "100";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal238328() {
        long decimalNumber = 238328;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "1000";
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimal14776336() {
        long decimalNumber = 14776336;
        String actual = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
        String expected = "10000";
        assertEquals(expected, actual);
    }

    @Test
    public void sixtytwoToDecimal0() {
        String decimalNumber = "0";
        long actual = SixtytwoNumberingSystem.sixtytwoToDecimal(decimalNumber);
        long expected = 0;
        assertEquals(expected, actual);
    }

    @Test
    public void sixtytwoToDecimalZ() {
        String decimalNumber = "Z";
        long actual = SixtytwoNumberingSystem.sixtytwoToDecimal(decimalNumber);
        long expected = 61;
        assertEquals(expected, actual);
    }

    @Test
    public void sixtytwoToDecimal10() {
        String decimalNumber = "10";
        long actual = SixtytwoNumberingSystem.sixtytwoToDecimal(decimalNumber);
        long expected = 62;
        assertEquals(expected, actual);
    }

    @Test
    public void sixtytwoToDecimal10000() {
        String decimalNumber = "10000";
        long actual = SixtytwoNumberingSystem.sixtytwoToDecimal(decimalNumber);
        long expected = 14776336;
        assertEquals(expected, actual);
    }

    @Test
    public void sixtytwoToDecimal1000() {
        String decimalNumber = "1000";
        long actual = SixtytwoNumberingSystem.sixtytwoToDecimal(decimalNumber);
        long expected = 238328;
        assertEquals(expected, actual);
    }

    @Test
    public void decimalToSixtytwoDecimalToDecimal() {
        for (long decimalNumber = 0; decimalNumber < 1000000L; ++ decimalNumber) {
            String sixtyTwoNumber = SixtytwoNumberingSystem.decimalToSixtytwo(decimalNumber);
            long newDecimalNumber = SixtytwoNumberingSystem.sixtytwoToDecimal(sixtyTwoNumber);
            assertEquals(decimalNumber, newDecimalNumber);
        }
    }

}
