package de.duehl.swing.ui.key;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.KeyStroke;

/**
 * Diese Klasse stellt Konstanten rund um die Belegung von Tastaturbefehlen für einen Dialog oder
 * Frame zur Verfügung. Dazu gehören bestimmte Strings und zugehörige Keystrokes.
 *
 * @version 1.01     2018-04-05
 * @author Christian Dühl
 */

public class KeybingingDefinitions {

    public static final String KEYBINDING_ESCAPE = "ESCAPE";
    public static final String KEYBINDING_POS1 = "HOME";
    public static final String KEYBINDING_END = "END";
    public static final String KEYBINDING_CTRL_POS1 = "CTRL_HOME";
    public static final String KEYBINDING_CTRL_END = "CTRL_END";
    public static final String KEYBINDING_PAGE_UP = "PAGE_UP";
    public static final String KEYBINDING_PAGE_DOWN = "PAGE_DOWN";
    public static final String KEYBINDING_ENTER = "ENTER";
    public static final String KEYBINDING_CTRL_ENTER = "CTRL_ENTER";

    public static final int KEYEVENT_ESCAPE = KeyEvent.VK_ESCAPE;
    public static final int KEYEVENT_POS1 = KeyEvent.VK_HOME;
    public static final int KEYEVENT_END = KeyEvent.VK_END;
    public static final int KEYEVENT_PAGE_UP = KeyEvent.VK_PAGE_UP;
    public static final int KEYEVENT_PAGE_DOWN = KeyEvent.VK_PAGE_DOWN;
    public static final int KEYEVENT_ENTER= KeyEvent.VK_ENTER;

    public static final KeyStroke KEYSTROKE_ESCAPE = createUnmodifiedKeystroke(KEYEVENT_ESCAPE);
    public static final KeyStroke KEYSTROKE_POS1 = createUnmodifiedKeystroke(KEYEVENT_POS1);
    public static final KeyStroke KEYSTROKE_END = createUnmodifiedKeystroke(KEYEVENT_END);
    public static final KeyStroke KEYSTROKE_PAGE_UP = createUnmodifiedKeystroke(KEYEVENT_PAGE_UP);
    public static final KeyStroke KEYSTROKE_PAGE_DOWN = createUnmodifiedKeystroke(KEYEVENT_PAGE_DOWN);
    public static final KeyStroke KEYSTROKE_ENTER = createUnmodifiedKeystroke(KEYEVENT_ENTER);

    public static final KeyStroke KEYSTROKE_CTRL_POS1 = createCtrlKeystroke(KEYEVENT_POS1);
    public static final KeyStroke KEYSTROKE_CTRL_END = createCtrlKeystroke(KEYEVENT_END);
    public static final KeyStroke KEYSTROKE_CTRL_ENTER = createCtrlKeystroke(KEYEVENT_ENTER);

    private static KeyStroke createUnmodifiedKeystroke(int keyevent) {
        return KeyStroke.getKeyStroke(keyevent, 0);
    }

    private static KeyStroke createCtrlKeystroke(int keyevent) {
        return KeyStroke.getKeyStroke(keyevent, InputEvent.CTRL_DOWN_MASK);
    }

}
